=== modified file 'BRANCH.TODO'
--- BRANCH.TODO	2007-03-01 12:20:44 +0000
+++ BRANCH.TODO	2007-03-05 03:43:56 +0000
@@ -15,6 +15,3 @@
 
 In things like get_file_sha1 on tree, consider a deprecation warning if
 path is not given? or some developer tool to profile.
-
-hashcache removal.
-

=== modified file 'NEWS'
--- NEWS	2007-03-01 05:00:53 +0000
+++ NEWS	2007-03-05 03:43:56 +0000
@@ -70,6 +70,18 @@
       "bzr unbind", you can "bzr bind" to bind to the previously-selected
       bind location.
 
+    * Builtin tags support, created and deleted by the ``tag`` command and
+      stored in the branch.  Tags can be accessed with the revisionspec
+      ``-rtag:``, and listed with ``bzr tags``.  Tags are not versioned 
+      at present.  (Martin Pool)
+
+    * The bzr:// transport now has a well-known port number, 4155, which it will
+      use by default.  (Andrew Bennetts, Martin Pool)
+
+    * Bazaar now looks for user-installed plugins before looking for site-wide
+      plugins. (Jonathan Lange)
+
+>>>>>>> MERGE-SOURCE
   INTERNALS:
 
     * Internally revision ids and file ids are now passed around as utf-8
@@ -118,14 +130,23 @@
       This is expected to grow to cover a number of related uses mainly
       related to bzr info. (Robert Collins)
 
-    * Log formatters are now managed with a registry.  log.register_formatter
-      continues to work, but callers accessed the FORMATTERS dictionary
-      directly will not.
+    * Log formatters are now managed with a registry.
+      ``log.register_formatter`` continues to work, but callers accessing
+      the FORMATTERS dictionary directly will not.
+
+    * Allow a start message to be passed to the ``edit_commit_message``
+      function.  This will be placed in the message offered to the user
+      for editing above the separator. It allows a template commit message
+      to be used more easily. (James Westby)
 
     * ``GPGStrategy.sign()`` will now raise ``BzrBadParameterUnicode`` if
       you pass a Unicode string rather than an 8-bit string. Callers need
       to be updated to encode first. (John Arbash Meinel)
 
+    * Branch.push, pull, merge now return Result objects with information
+      about what happened, rather than a scattering of various methods.  These
+      are also passed to the post hooks.  (Martin Pool)
+
   BUGFIXES:
 
     * ``bzr annotate`` now uses dotted revnos from the viewpoint of the
@@ -141,7 +162,7 @@
       (John Arbash Meinel, #30576, #45504)
 
     * Fix http proxy authentication when user and an optional
-      password appears in the *_proxy vars. (Vincent Ladeuil,
+      password appears in the ``*_proxy`` vars. (Vincent Ladeuil,
       #83954).
 
     * ``bzr log branch/file`` works for local treeless branches
@@ -149,6 +170,15 @@
 
     * Fix problem with UNC paths on Windows 98. (Alexander Belchenko, #84728)
 
+    * Searching location of CA bundle for PyCurl in env variable (CURL_CA_BUNDLE),
+      and on win32 along the PATH. (Alexander Belchenko, #82086)
+
+    * ``bzr init`` works with unicode argument LOCATION.
+      (Alexander Belchenko, #85599)
+
+    * Raise DependencyNotPresent if pycurl do not support https. (Vincent
+      Ladeuil, #85305)
+
   TESTING:
 
     * New ``--first`` option to ``bzr selftest`` to run specified tests
@@ -302,6 +332,8 @@
     * Allow plugins import from zip archives.
       (Alexander Belchenko, #68124)
 
+    * bzr rocks. It sure does! Fix case. (Vincent Ladeuil, #78026)
+
 
 bzr 0.13  2006-12-05
     

=== modified file 'bzr'
--- bzr	2007-02-20 07:16:27 +0000
+++ bzr	2007-02-28 18:39:16 +0000
@@ -104,7 +104,12 @@
     # are just about to be discarded anyhow.  This does mean that atexit hooks
     # won't run but we don't use them.  Also file buffers won't be flushed,
     # but our policy is to always close files from a finally block. -- mbp 20070215
-    sys.stdout.flush()
+    try:
+        sys.stdout.flush()
+    except IOError, e:
+        import errno
+        if sys.platform != 'win32' or e.errno != errno.EINVAL:
+            raise
     if bzrlib.trace._trace_file:
         # this is also _bzr_log
         bzrlib.trace._trace_file.flush()

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-03-01 16:01:33 +0000
+++ bzrlib/branch.py	2007-03-05 03:43:56 +0000
@@ -40,6 +40,10 @@
         )
 from bzrlib.config import BranchConfig, TreeConfig
 from bzrlib.lockable_files import LockableFiles, TransportLock
+from bzrlib.tag import (
+    BasicTags,
+    DisabledTags,
+    )
 """)
 
 from bzrlib.decorators import needs_read_lock, needs_write_lock
@@ -88,8 +92,12 @@
     # - RBC 20060112
     base = None
 
+    # override this to set the strategy for storing tags
+    def _make_tags(self):
+        return DisabledTags(self)
+
     def __init__(self, *ignored, **ignored_too):
-        raise NotImplementedError('The Branch class is abstract')
+        self.tags = self._make_tags()
 
     def break_lock(self):
         """Break a lock if one is present from another instance.
@@ -399,6 +407,8 @@
         """Mirror source into this branch.
 
         This branch is considered to be 'local', having low latency.
+
+        :returns: PullResult instance
         """
         raise NotImplementedError(self.pull)
 
@@ -677,6 +687,9 @@
             checkout_branch.pull(self, stop_revision=revision_id)
         return checkout.create_workingtree(revision_id)
 
+    def supports_tags(self):
+        return self._format.supports_tags()
+
 
 class BranchFormat(object):
     """An encapsulation of the initialization and open routines for a format.
@@ -796,6 +809,24 @@
     def __str__(self):
         return self.get_format_string().rstrip()
 
+    def supports_tags(self):
+        """True if this format supports tags stored in the branch"""
+        return False  # by default
+
+    # XXX: Probably doesn't really belong here -- mbp 20070212
+    def _initialize_control_files(self, a_bzrdir, utf8_files, lock_filename,
+            lock_class):
+        branch_transport = a_bzrdir.get_branch_transport(self)
+        control_files = lockable_files.LockableFiles(branch_transport,
+            lock_filename, lock_class)
+        control_files.create_lock()
+        control_files.lock_write()
+        try:
+            for filename, content in utf8_files:
+                control_files.put_utf8(filename, content)
+        finally:
+            control_files.unlock()
+
 
 class BranchHooks(dict):
     """A dictionary mapping hook name to a list of callables for branch hooks.
@@ -819,6 +850,8 @@
         self['set_rh'] = []
         # invoked after a push operation completes.
         # the api signature is
+        # (push_result)
+        # containing the members
         # (source, local, master, old_revno, old_revid, new_revno, new_revid)
         # where local is the local branch or None, master is the target 
         # master branch, and the rest should be self explanatory. The source
@@ -827,6 +860,8 @@
         self['post_push'] = []
         # invoked after a pull operation completes.
         # the api signature is
+        # (pull_result)
+        # containing the members
         # (source, local, master, old_revno, old_revid, new_revno, new_revid)
         # where local is the local branch or None, master is the target 
         # master branch, and the rest should be self explanatory. The source
@@ -955,9 +990,6 @@
                           a_bzrdir=a_bzrdir,
                           _repository=a_bzrdir.find_repository())
 
-    def __str__(self):
-        return "Bazaar-NG Metadir branch format 5"
-
 
 class BzrBranchFormat6(BzrBranchFormat5):
     """Branch format with last-revision
@@ -981,7 +1013,8 @@
         """Create a branch of this format in a_bzrdir."""
         utf8_files = [('last-revision', '0 null:\n'),
                       ('branch-name', ''),
-                      ('branch.conf', '')
+                      ('branch.conf', ''),
+                      ('tags', ''),
                       ]
         return self._initialize_helper(a_bzrdir, utf8_files)
 
@@ -1002,6 +1035,9 @@
                           a_bzrdir=a_bzrdir,
                           _repository=a_bzrdir.find_repository())
 
+    def supports_tags(self):
+        return True
+
 
 class BranchReferenceFormat(BranchFormat):
     """Bzr branch reference format.
@@ -1107,16 +1143,20 @@
             upgrade/recovery type use; it's not guaranteed that
             all operations will work on old format branches.
         """
+        Branch.__init__(self)
         if a_bzrdir is None:
             self.bzrdir = bzrdir.BzrDir.open(transport.base)
         else:
             self.bzrdir = a_bzrdir
-        self._transport = self.bzrdir.transport.clone('..')
-        self._base = self._transport.base
+        # self._transport used to point to the directory containing the
+        # control directory, but was not used - now it's just the transport
+        # for the branch control files.  mbp 20070212
+        self._base = self.bzrdir.transport.clone('..').base
         self._format = _format
         if _control_files is None:
             raise ValueError('BzrBranch _control_files is None')
         self.control_files = _control_files
+        self._transport = _control_files._transport
         if deprecated_passed(init):
             warn("BzrBranch.__init__(..., init=XXX): The init parameter is "
                  "deprecated as of bzr 0.8. Please use Branch.create().",
@@ -1151,6 +1191,7 @@
     __repr__ = __str__
 
     def _get_base(self):
+        """Returns the directory containing the control directory."""
         return self._base
 
     base = property(_get_base, doc="The URL for the root of this branch.")
@@ -1397,29 +1438,35 @@
         :param _run_hooks: Private parameter - allow disabling of
             hooks, used when pushing to a master branch.
         """
+        result = PullResult()
+        result.source_branch = source
+        result.target_branch = self
         source.lock_read()
         try:
-            old_count, old_tip = self.last_revision_info()
+            result.old_revno, result.old_revid = self.last_revision_info()
             try:
                 self.update_revisions(source, stop_revision)
             except DivergedBranches:
                 if not overwrite:
                     raise
             if overwrite:
-                self.set_revision_history(source.revision_history())
-            new_count, new_tip = self.last_revision_info()
+                if stop_revision is None:
+                    stop_revision = source.last_revision()
+                self.generate_revision_history(stop_revision)
+            result.tag_conflicts = source.tags.merge_to(self.tags)
+            result.new_revno, result.new_revid = self.last_revision_info()
+            if _hook_master:
+                result.master_branch = _hook_master
+                result.local_branch = self
+            else:
+                result.master_branch = self
+                result.local_branch = None
             if _run_hooks:
-                if _hook_master:
-                    _hook_local = self
-                else:
-                    _hook_master = self
-                    _hook_local = None
                 for hook in Branch.hooks['post_pull']:
-                    hook(source, _hook_local, _hook_master, old_count, old_tip,
-                        new_count, new_tip)
-            return new_count - old_count
+                    hook(result)
         finally:
             source.unlock()
+        return result
 
     def _get_parent_location(self):
         _locs = ['parent', 'pull', 'x-pull']
@@ -1440,9 +1487,12 @@
         :param _run_hooks: Private parameter - allow disabling of
             hooks, used when pushing to a master branch.
         """
+        result = PushResult()
+        result.source_branch = self
+        result.target_branch = target
         target.lock_write()
         try:
-            old_count, old_tip = target.last_revision_info()
+            result.old_revno, result.old_revid = target.last_revision_info()
             try:
                 target.update_revisions(self, stop_revision)
             except DivergedBranches:
@@ -1450,19 +1500,20 @@
                     raise
             if overwrite:
                 target.set_revision_history(self.revision_history())
-            new_count, new_tip = target.last_revision_info()
+            result.tag_conflicts = self.tags.merge_to(target.tags)
+            result.new_revno, result.new_revid = target.last_revision_info()
+            if _hook_master:
+                result.master_branch = _hook_master
+                result.local_branch = target
+            else:
+                result.master_branch = target
+                result.local_branch = None
             if _run_hooks:
-                if _hook_master:
-                    _hook_local = target
-                else:
-                    _hook_master = target
-                    _hook_local = None
                 for hook in Branch.hooks['post_push']:
-                    hook(self, _hook_local, _hook_master, old_count, old_tip,
-                        new_count, new_tip)
-            return new_count - old_count
+                    hook(result)
         finally:
             target.unlock()
+        return result
 
     def get_parent(self):
         """See Branch.get_parent."""
@@ -1694,6 +1745,94 @@
         return None
 
 
+class BzrBranchExperimental(BzrBranch5):
+    """Bzr experimental branch format
+
+    This format has:
+     - a revision-history file.
+     - a format string
+     - a lock dir guarding the branch itself
+     - all of this stored in a branch/ subdirectory
+     - works with shared repositories.
+     - a tag dictionary in the branch
+
+    This format is new in bzr 0.15, but shouldn't be used for real data, 
+    only for testing.
+
+    This class acts as it's own BranchFormat.
+    """
+
+    _matchingbzrdir = bzrdir.BzrDirMetaFormat1()
+
+    @classmethod
+    def get_format_string(cls):
+        """See BranchFormat.get_format_string()."""
+        return "Bazaar-NG branch format experimental\n"
+
+    @classmethod
+    def get_format_description(cls):
+        """See BranchFormat.get_format_description()."""
+        return "Experimental branch format"
+
+    @classmethod
+    def _initialize_control_files(cls, a_bzrdir, utf8_files, lock_filename,
+            lock_class):
+        branch_transport = a_bzrdir.get_branch_transport(cls)
+        control_files = lockable_files.LockableFiles(branch_transport,
+            lock_filename, lock_class)
+        control_files.create_lock()
+        control_files.lock_write()
+        try:
+            for filename, content in utf8_files:
+                control_files.put_utf8(filename, content)
+        finally:
+            control_files.unlock()
+        
+    @classmethod
+    def initialize(cls, a_bzrdir):
+        """Create a branch of this format in a_bzrdir."""
+        utf8_files = [('format', cls.get_format_string()),
+                      ('revision-history', ''),
+                      ('branch-name', ''),
+                      ('tags', ''),
+                      ]
+        cls._initialize_control_files(a_bzrdir, utf8_files,
+            'lock', lockdir.LockDir)
+        return cls.open(a_bzrdir, _found=True)
+
+    @classmethod
+    def open(cls, a_bzrdir, _found=False):
+        """Return the branch object for a_bzrdir
+
+        _found is a private parameter, do not use it. It is used to indicate
+               if format probing has already be done.
+        """
+        if not _found:
+            format = BranchFormat.find_format(a_bzrdir)
+            assert format.__class__ == cls
+        transport = a_bzrdir.get_branch_transport(None)
+        control_files = lockable_files.LockableFiles(transport, 'lock',
+                                                     lockdir.LockDir)
+        return cls(_format=cls,
+            _control_files=control_files,
+            a_bzrdir=a_bzrdir,
+            _repository=a_bzrdir.find_repository())
+
+    @classmethod
+    def is_supported(cls):
+        return True
+
+    def _make_tags(self):
+        return BasicTags(self)
+
+    @classmethod
+    def supports_tags(cls):
+        return True
+
+
+BranchFormat.register_format(BzrBranchExperimental)
+
+
 class BzrBranch6(BzrBranch5):
 
     @needs_read_lock
@@ -1882,6 +2021,9 @@
             revno = self.revision_id_to_revno(revision_id)
         destination.set_last_revision_info(revno, revision_id)
 
+    def _make_tags(self):
+        return BasicTags(self)
+
 
 class BranchTestProviderAdapter(object):
     """A tool to generate a suite testing multiple branch formats at once.
@@ -1906,13 +2048,79 @@
             new_test.bzrdir_format = bzrdir_format
             new_test.branch_format = branch_format
             def make_new_test_id():
-                new_id = "%s(%s)" % (new_test.id(), branch_format.__class__.__name__)
+                # the format can be either a class or an instance
+                name = getattr(branch_format, '__name__',
+                        branch_format.__class__.__name__)
+                new_id = "%s(%s)" % (new_test.id(), name)
                 return lambda: new_id
             new_test.id = make_new_test_id()
             result.addTest(new_test)
         return result
 
 
+######################################################################
+# results of operations
+
+
+class _Result(object):
+
+    def _show_tag_conficts(self, to_file):
+        if not getattr(self, 'tag_conflicts', None):
+            return
+        to_file.write('Conflicting tags:\n')
+        for name, value1, value2 in self.tag_conflicts:
+            to_file.write('    %s\n' % (name, ))
+
+
+class PullResult(_Result):
+    """Result of a Branch.pull operation.
+
+    :ivar old_revno: Revision number before pull.
+    :ivar new_revno: Revision number after pull.
+    :ivar old_revid: Tip revision id before pull.
+    :ivar new_revid: Tip revision id after pull.
+    :ivar source_branch: Source (local) branch object.
+    :ivar master_branch: Master branch of the target, or None.
+    :ivar target_branch: Target/destination branch object.
+    """
+
+    def __int__(self):
+        # DEPRECATED: pull used to return the change in revno
+        return self.new_revno - self.old_revno
+
+    def report(self, to_file):
+        if self.old_revid == self.new_revid:
+            to_file.write('No revisions to pull.\n')
+        else:
+            to_file.write('Now on revision %d.\n' % self.new_revno)
+        self._show_tag_conficts(to_file)
+
+
+class PushResult(_Result):
+    """Result of a Branch.push operation.
+
+    :ivar old_revno: Revision number before push.
+    :ivar new_revno: Revision number after push.
+    :ivar old_revid: Tip revision id before push.
+    :ivar new_revid: Tip revision id after push.
+    :ivar source_branch: Source branch object.
+    :ivar master_branch: Master branch of the target, or None.
+    :ivar target_branch: Target/destination branch object.
+    """
+
+    def __int__(self):
+        # DEPRECATED: push used to return the change in revno
+        return self.new_revno - self.old_revno
+
+    def report(self, to_file):
+        """Write a human-readable description of the result."""
+        if self.old_revid == self.new_revid:
+            to_file.write('No new revisions to push.\n')
+        else:
+            to_file.write('Pushed up to revision %d.\n' % self.new_revno)
+        self._show_tag_conficts(to_file)
+
+
 class BranchCheckResult(object):
     """Results of checking branch consistency.
 
@@ -1933,17 +2141,6 @@
              self.branch._format)
 
 
-######################################################################
-# predicates
-
-
-@deprecated_function(zero_eight)
-def is_control_file(*args, **kwargs):
-    """See bzrlib.workingtree.is_control_file."""
-    from bzrlib import workingtree
-    return workingtree.is_control_file(*args, **kwargs)
-
-
 class Converter5to6(object):
     """Perform an in-place upgrade of format 5 to format 6"""
 
@@ -1958,6 +2155,9 @@
         new_branch.set_bound_location(branch.get_bound_location())
         new_branch.set_push_location(branch.get_push_location())
 
+        # New branch has no tags by default
+        new_branch.tags._set_tag_dict({})
+
         # Copying done; now update target format
         new_branch.control_files.put_utf8('format',
             format.get_format_string())

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2007-03-05 01:15:25 +0000
+++ bzrlib/builtins.py	2007-03-05 03:43:56 +0000
@@ -559,6 +559,7 @@
     def run(self, location=None, remember=False, overwrite=False,
             revision=None, verbose=False,
             directory=None):
+        from bzrlib.tag import _merge_tags_if_possible
         # FIXME: too much stuff is in the command class
         if directory is None:
             directory = u'.'
@@ -607,19 +608,16 @@
 
         old_rh = branch_to.revision_history()
         if tree_to is not None:
-            count = tree_to.pull(branch_from, overwrite, rev_id,
-                delta.ChangeReporter())
+            result = tree_to.pull(branch_from, overwrite, rev_id,
+                delta.ChangeReporter(unversioned_filter=tree_to.is_ignored))
         else:
-            count = branch_to.pull(branch_from, overwrite, rev_id)
-        note('%d revision(s) pulled.' % (count,))
+            result = branch_to.pull(branch_from, overwrite, rev_id)
 
+        result.report(self.outf)
         if verbose:
+            from bzrlib.log import show_changed_revisions
             new_rh = branch_to.revision_history()
-            if old_rh != new_rh:
-                # Something changed
-                from bzrlib.log import show_changed_revisions
-                show_changed_revisions(branch_to, old_rh, new_rh,
-                                       to_file=self.outf)
+            show_changed_revisions(branch_to, old_rh, new_rh, to_file=self.outf)
 
 
 class cmd_push(Command):
@@ -689,9 +687,6 @@
         to_transport = transport.get_transport(location)
         location_url = to_transport.base
 
-        old_rh = []
-        count = 0
-
         br_to = repository_to = dir_to = None
         try:
             dir_to = bzrdir.BzrDir.open_from_transport(to_transport)
@@ -711,9 +706,10 @@
             else:
                 # Found a branch, so we must have found a repository
                 repository_to = br_to.repository
-
+        push_result = None
         old_rh = []
         if dir_to is None:
+            # The destination doesn't exist; create it.
             # XXX: Refactor the create_prefix/no_create_prefix code into a
             #      common helper function
             try:
@@ -760,7 +756,8 @@
             dir_to = br_from.bzrdir.clone(location_url,
                 revision_id=br_from.last_revision())
             br_to = dir_to.open_branch()
-            count = br_to.last_revision_info()[0]
+            # TODO: Some more useful message about what was copied
+            note('Created new branch.')
             # We successfully created the target, remember it
             if br_from.get_push_location() is None or remember:
                 br_from.set_push_location(br_to.base)
@@ -779,7 +776,7 @@
             repository_to.fetch(br_from.repository,
                                 revision_id=last_revision_id)
             br_to = br_from.clone(dir_to, revision_id=last_revision_id)
-            count = len(br_to.revision_history())
+            note('Created new branch.')
             if br_from.get_push_location() is None or remember:
                 br_from.set_push_location(br_to.base)
         else: # We have a valid to branch
@@ -794,28 +791,32 @@
                 except errors.NotLocalUrl:
                     warning('This transport does not update the working '
                             'tree of: %s' % (br_to.base,))
-                    count = br_from.push(br_to, overwrite)
+                    push_result = br_from.push(br_to, overwrite)
                 except errors.NoWorkingTree:
-                    count = br_from.push(br_to, overwrite)
+                    push_result = br_from.push(br_to, overwrite)
                 else:
                     tree_to.lock_write()
                     try:
-                        count = br_from.push(tree_to.branch, overwrite)
+                        push_result = br_from.push(tree_to.branch, overwrite)
                         tree_to.update()
                     finally:
                         tree_to.unlock()
             except errors.DivergedBranches:
                 raise errors.BzrCommandError('These branches have diverged.'
                                         '  Try using "merge" and then "push".')
-        note('%d revision(s) pushed.' % (count,))
-
-        if verbose:
+        if push_result is not None:
+            push_result.report(self.outf)
+        elif verbose:
             new_rh = br_to.revision_history()
             if old_rh != new_rh:
                 # Something changed
                 from bzrlib.log import show_changed_revisions
                 show_changed_revisions(br_to, old_rh, new_rh,
                                        to_file=self.outf)
+        else:
+            # we probably did a clone rather than a push, so a message was
+            # emitted above
+            pass
 
 
 class cmd_branch(Command):
@@ -836,6 +837,7 @@
     aliases = ['get', 'clone']
 
     def run(self, from_location, to_location=None, revision=None, basis=None):
+        from bzrlib.tag import _merge_tags_if_possible
         if revision is None:
             revision = [None]
         elif len(revision) > 1:
@@ -886,6 +888,7 @@
                 raise errors.BzrCommandError(msg)
             if name:
                 branch.control_files.put_utf8('branch-name', name)
+            _merge_tags_if_possible(br_from, branch)
             note('Branched %d revision(s).' % branch.revno())
         finally:
             br_from.unlock()
@@ -1253,7 +1256,7 @@
             existing_bzrdir = bzrdir.BzrDir.open(location)
         except errors.NotBranchError:
             # really a NotBzrDir error...
-            branch = bzrdir.BzrDir.create_branch_convenience(location,
+            branch = bzrdir.BzrDir.create_branch_convenience(to_transport.base,
                                                              format=format)
         else:
             from bzrlib.transport.local import LocalTransport
@@ -1964,15 +1967,14 @@
 
         tree = None
         try:
-            tree, relpath = WorkingTree.open_containing(filename)
-            b = tree.branch
-        except (errors.NotBranchError, errors.NotLocalUrl):
+            tree, b, relpath = \
+                    bzrdir.BzrDir.open_containing_tree_or_branch(filename)
+        except errors.NotBranchError:
             pass
 
         if revision is not None and revision[0].get_branch() is not None:
             b = Branch.open(revision[0].get_branch())
         if tree is None:
-            b, relpath = Branch.open_containing(filename)
             tree = b.basis_tree()
         if revision is None:
             revision_id = b.last_revision()
@@ -2371,7 +2373,7 @@
 
     @display_command
     def run(self):
-        print "it sure does!"
+        print "It sure does!"
 
 
 class cmd_find_merge_base(Command):
@@ -2455,7 +2457,7 @@
                 ' source rather than merging. When this happens,'
                 ' you do not need to commit the result.'),
         Option('directory',
-            help='branch to merge into, '
+            help='Branch to merge into, '
                  'rather than the one containing the working directory',
             short_name='d',
             type=unicode,
@@ -2467,6 +2469,7 @@
             uncommitted=False, pull=False,
             directory=None,
             ):
+        from bzrlib.tag import _merge_tags_if_possible
         if merge_type is None:
             merge_type = _mod_merge.Merge3Merger
 
@@ -2479,7 +2482,8 @@
         #      Either the merge helper code should be updated to take a tree,
         #      (What about tree.merge_from_branch?)
         tree = WorkingTree.open_containing(directory)[0]
-        change_reporter = delta.ChangeReporter()
+        change_reporter = delta.ChangeReporter(
+            unversioned_filter=tree.is_ignored)
 
         if branch is not None:
             try:
@@ -2534,6 +2538,10 @@
         if tree.branch.get_parent() is None or remember:
             tree.branch.set_parent(other_branch.base)
 
+        # pull tags now... it's a bit inconsistent to do it ahead of copying
+        # the history but that's done inside the merge code
+        _merge_tags_if_possible(other_branch, tree.branch)
+
         if path != "":
             interesting_files = [path]
         else:
@@ -3174,7 +3182,8 @@
         Option('port',
                help='listen for connections on nominated port of the form '
                     '[hostname:]portnumber. Passing 0 as the port number will '
-                    'result in a dynamically allocated port.',
+                    'result in a dynamically allocated port. Default port is '
+                    '4155.',
                type=str),
         Option('directory',
                help='serve contents of directory',
@@ -3197,19 +3206,104 @@
         t = get_transport(url)
         if inet:
             server = smart.SmartServerPipeStreamMedium(sys.stdin, sys.stdout, t)
-        elif port is not None:
-            if ':' in port:
-                host, port = port.split(':')
-            else:
+        else:
+            if port is None:
+                port = smart.BZR_DEFAULT_PORT
                 host = '127.0.0.1'
-            server = smart.SmartTCPServer(t, host=host, port=int(port))
+            else:
+                if ':' in port:
+                    host, port = port.split(':')
+                else:
+                    host = '127.0.0.1'
+                port = int(port)
+            server = smart.SmartTCPServer(t, host=host, port=port)
             print 'listening on port: ', server.port
             sys.stdout.flush()
-        else:
-            raise errors.BzrCommandError("bzr serve requires one of --inet or --port")
         server.serve()
 
 
+class cmd_tag(Command):
+    """Create a tag naming a revision.
+    
+    Tags give human-meaningful names to revisions.  Commands that take a -r
+    (--revision) option can be given -rtag:X, where X is any previously
+    created tag.
+
+    Tags are stored in the branch.  Tags are copied from one branch to another
+    along when you branch, push, pull or merge.
+
+    It is an error to give a tag name that already exists unless you pass 
+    --force, in which case the tag is moved to point to the new revision.
+    """
+
+    takes_args = ['tag_name']
+    takes_options = [
+        Option('delete',
+            help='Delete this tag rather than placing it.',
+            ),
+        Option('directory',
+            help='Branch in which to place the tag.',
+            short_name='d',
+            type=unicode,
+            ),
+        Option('force',
+            help='Replace existing tags',
+            ),
+        'revision',
+        ]
+
+    def run(self, tag_name,
+            delete=None,
+            directory='.',
+            force=None,
+            revision=None,
+            ):
+        branch, relpath = Branch.open_containing(directory)
+        branch.lock_write()
+        try:
+            if delete:
+                branch.tags.delete_tag(tag_name)
+                self.outf.write('Deleted tag %s.\n' % tag_name)
+            else:
+                if revision:
+                    if len(revision) != 1:
+                        raise errors.BzrCommandError(
+                            "Tags can only be placed on a single revision, "
+                            "not on a range")
+                    revision_id = revision[0].in_history(branch).rev_id
+                else:
+                    revision_id = branch.last_revision()
+                if (not force) and branch.tags.has_tag(tag_name):
+                    raise errors.TagAlreadyExists(tag_name)
+                branch.tags.set_tag(tag_name, revision_id)
+                self.outf.write('Created tag %s.\n' % tag_name)
+        finally:
+            branch.unlock()
+
+
+class cmd_tags(Command):
+    """List tags.
+
+    This tag shows a table of tag names and the revisions they reference.
+    """
+
+    takes_options = [
+        Option('directory',
+            help='Branch whose tags should be displayed',
+            short_name='d',
+            type=unicode,
+            ),
+    ]
+
+    @display_command
+    def run(self,
+            directory='.',
+            ):
+        branch, relpath = Branch.open_containing(directory)
+        for tag_name, target in sorted(branch.tags.get_tag_dict().items()):
+            self.outf.write('%-20s %s\n' % (tag_name, target))
+
+
 # command-line interpretation helper for merge-related commands
 def _merge_helper(other_revision, base_revision,
                   check_clean=True, ignore_zero=False,
@@ -3282,9 +3376,13 @@
             return 0
         if file_list is None:
             if pull and merger.base_rev_id == merger.this_rev_id:
-                count = merger.this_tree.pull(merger.this_branch,
+                # FIXME: deduplicate with pull
+                result = merger.this_tree.pull(merger.this_branch,
                         False, merger.other_rev_id)
-                note('%d revision(s) pulled.' % (count,))
+                if result.old_revid == result.new_revid:
+                    note('No revisions to pull.')
+                else:
+                    note('Now on revision %d.' % result.new_revno)
                 return 0
         merger.backup_files = backup_files
         merger.merge_type = merge_type 

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2006-12-19 17:37:38 +0000
+++ bzrlib/commands.py	2007-02-28 18:39:16 +0000
@@ -627,7 +627,7 @@
                 raise
             if e.errno != errno.EPIPE:
                 # Win32 raises IOError with errno=0 on a broken pipe
-                if sys.platform != 'win32' or e.errno != 0:
+                if sys.platform != 'win32' or (e.errno not in (0, errno.EINVAL)):
                     raise
             pass
         except KeyboardInterrupt:

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-02-26 22:08:14 +0000
+++ bzrlib/errors.py	2007-03-05 03:43:56 +0000
@@ -1812,3 +1812,27 @@
     def __init__(self, name):
         BzrError.__init__(self)
         self.name = name
+
+
+class NoSuchTag(BzrError):
+
+    _fmt = "No such tag: %(tag_name)s"
+
+    def __init__(self, tag_name):
+        self.tag_name = tag_name
+
+
+class TagsNotSupported(BzrError):
+
+    _fmt = "Tags not supported by %(branch)s; you may be able to use bzr upgrade."
+
+    def __init__(self, branch):
+        self.branch = branch
+
+        
+class TagAlreadyExists(BzrError):
+
+    _fmt = "Tag %(tag_name)s already exists."
+
+    def __init__(self, tag_name):
+        self.tag_name = tag_name

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2007-02-28 15:41:01 +0000
+++ bzrlib/inventory.py	2007-03-05 03:43:56 +0000
@@ -956,8 +956,8 @@
         :return: This yields (path, entry) pairs
         """
         if specific_file_ids:
-            specific_file_ids = [osutils.safe_file_id(fid)
-                                 for fid in specific_file_ids]
+            safe = osutils.safe_file_id
+            specific_file_ids = set(safe(fid) for fid in specific_file_ids)
         # TODO? Perhaps this should return the from_dir so that the root is
         # yielded? or maybe an option?
         if from_dir is None:

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2006-11-11 19:25:25 +0000
+++ bzrlib/msgeditor.py	2007-02-26 21:48:05 +0000
@@ -80,7 +80,8 @@
     { 'bar' : '-' * 14, 'msg' : 'This line and the following will be ignored' }
 
 
-def edit_commit_message(infotext, ignoreline=DEFAULT_IGNORE_LINE):
+def edit_commit_message(infotext, ignoreline=DEFAULT_IGNORE_LINE,
+                        start_message=None):
     """Let the user edit a commit message in a temp file.
 
     This is run if they don't give a message or
@@ -90,21 +91,34 @@
         Text to be displayed at bottom of message for
         the user's reference; currently similar to
         'bzr status'.
+
+    ignoreline:
+        The separator to use above the infotext.
+
+    start_message:
+        The text to place above the separator, if any. This will not be
+        removed from the message after the user has edited it.
     """
     import tempfile
 
     msgfilename = None
     try:
         tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.', dir=u'.')
-        msgfile = os.close(tmp_fileno)
-        if infotext is not None and infotext != "":
-            hasinfo = True
-            msgfile = file(msgfilename, "w")
-            msgfile.write("\n\n%s\n\n%s" % (ignoreline,
-                infotext.encode(bzrlib.user_encoding, 'replace')))
+        msgfile = os.fdopen(tmp_fileno, 'w')
+        try:
+            if start_message is not None:
+                msgfile.write("%s\n" % start_message.encode(
+                                           bzrlib.user_encoding, 'replace'))
+
+            if infotext is not None and infotext != "":
+                hasinfo = True
+                msgfile.write("\n\n%s\n\n%s" % (ignoreline,
+                              infotext.encode(bzrlib.user_encoding,
+                                                    'replace')))
+            else:
+                hasinfo = False
+        finally:
             msgfile.close()
-        else:
-            hasinfo = False
 
         if not _run_editor(msgfilename):
             return None

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2007-02-02 20:27:53 +0000
+++ bzrlib/plugin.py	2007-02-23 06:06:54 +0000
@@ -84,8 +84,8 @@
     """Set the path for plugins to be loaded from."""
     path = os.environ.get('BZR_PLUGIN_PATH',
                           get_default_plugin_path()).split(os.pathsep)
-    # search the bzrlib installed dir before anything else.
-    path.insert(0, os.path.dirname(plugins.__file__))
+    # search the plugin path before the bzrlib installed dir
+    path.append(os.path.dirname(plugins.__file__))
     plugins.__path__ = path
     return path
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-02-22 15:04:35 +0000
+++ bzrlib/repository.py	2007-03-05 03:43:56 +0000
@@ -60,6 +60,9 @@
 _deprecation_warning_done = False
 
 
+######################################################################
+# Repositories
+
 class Repository(object):
     """Repository holding history for one or more branches.
 
@@ -1036,6 +1039,9 @@
 """
 
 
+#####################################################################
+# Repository Formats
+
 class RepositoryFormat(object):
     """A repository format.
 
@@ -1133,7 +1139,7 @@
         from bzrlib.store.revision.text import TextRevisionStore
         dir_mode = control_files._dir_mode
         file_mode = control_files._file_mode
-        text_store =TextStore(transport.clone(name),
+        text_store = TextStore(transport.clone(name),
                               prefixed=prefixed,
                               compressed=compressed,
                               dir_mode=dir_mode,

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2007-02-01 14:47:47 +0000
+++ bzrlib/revisionspec.py	2007-02-20 07:50:00 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -93,6 +93,18 @@
         return '<bzrlib.revisionspec.RevisionInfo object %s, %s for %r>' % (
             self.revno, self.rev_id, self.branch)
 
+    @staticmethod
+    def from_revision_id(branch, revision_id, revs):
+        """Construct a RevisionInfo given just the id.
+
+        Use this if you don't know or care what the revno is.
+        """
+        try:
+            revno = revs.index(revision_id) + 1
+        except ValueError:
+            revno = None
+        return RevisionInfo(branch, revno, revision_id)
+
 
 # classes in this list should have a "prefix" attribute, against which
 # string specs are matched
@@ -367,11 +379,7 @@
     prefix = 'revid:'
 
     def _match_on(self, branch, revs):
-        try:
-            revno = revs.index(self.spec) + 1
-        except ValueError:
-            revno = None
-        return RevisionInfo(branch, revno, self.spec)
+        return RevisionInfo.from_revision_id(branch, self.spec, revs)
 
 SPEC_TYPES.append(RevisionSpec_revid)
 
@@ -465,16 +473,21 @@
 
 
 class RevisionSpec_tag(RevisionSpec):
-    """To be implemented."""
-
-    help_txt = """To be implemented."""
+    """Select a revision identified by tag name"""
+
+    help_txt = """Selects a revision identified by a tag name.
+
+    Tags are stored in the repository and created by the 'tag'
+    command.
+    """
 
     prefix = 'tag:'
 
     def _match_on(self, branch, revs):
-        raise errors.InvalidRevisionSpec(self.user_spec, branch,
-                                         'tag: namespace registered,'
-                                         ' but not implemented')
+        # Can raise tags not supported, NoSuchTag, etc
+        return RevisionInfo.from_revision_id(branch,
+            branch.tags.lookup_tag(self.spec),
+            revs)
 
 SPEC_TYPES.append(RevisionSpec_tag)
 

=== added file 'bzrlib/tag.py'
--- bzrlib/tag.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tag.py	2007-02-26 05:29:05 +0000
@@ -0,0 +1,222 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tag strategies.
+
+These are contained within a branch and normally constructed 
+when the branch is opened.  Clients should typically do 
+
+  Branch.tags.add('name', 'value')
+"""
+
+# NOTE: I was going to call this tags.py, but vim seems to think all files
+# called tags* are ctags files... mbp 20070220.
+
+
+from warnings import warn
+
+from bzrlib import (
+    errors,
+    trace,
+    )
+from bzrlib.util import bencode
+
+
+class _Tags(object):
+
+    def __init__(self, branch):
+        self.branch = branch
+
+    def has_tag(self, tag_name):
+        return self.get_tag_dict().has_key(tag_name)
+
+
+class DisabledTags(_Tags):
+    """Tag storage that refuses to store anything.
+
+    This is used by older formats that can't store tags.
+    """
+
+    def _not_supported(self, *a, **k):
+        raise errors.TagsNotSupported(self.branch)
+
+    def supports_tags(self):
+        return False
+
+    set_tag = _not_supported
+    get_tag_dict = _not_supported
+    _set_tag_dict = _not_supported
+    lookup_tag = _not_supported
+    delete_tag = _not_supported
+
+    def merge_to(self, to_tags):
+        # we never have anything to copy
+        pass
+
+
+class BasicTags(_Tags):
+    """Tag storage in an unversioned branch control file.
+    """
+
+    def supports_tags(self):
+        return True
+
+    def set_tag(self, tag_name, tag_target):
+        """Add a tag definition to the branch.
+
+        Behaviour if the tag is already present is not defined (yet).
+        """
+        # all done with a write lock held, so this looks atomic
+        self.branch.lock_write()
+        try:
+            td = self.get_tag_dict()
+            td[tag_name] = tag_target
+            self._set_tag_dict(td)
+        finally:
+            self.branch.unlock()
+
+    def lookup_tag(self, tag_name):
+        """Return the referent string of a tag"""
+        td = self.get_tag_dict()
+        try:
+            return td[tag_name]
+        except KeyError:
+            raise errors.NoSuchTag(tag_name)
+
+    def get_tag_dict(self):
+        self.branch.lock_read()
+        try:
+            try:
+                tag_content = self.branch._transport.get_bytes('tags')
+            except errors.NoSuchFile, e:
+                # ugly, but only abentley should see this :)
+                trace.warning('No branch/tags file in %s.  '
+                     'This branch was probably created by bzr 0.15pre.  '
+                     'Create an empty file to silence this message.'
+                     % (self.branch, ))
+                return {}
+            return self._deserialize_tag_dict(tag_content)
+        finally:
+            self.branch.unlock()
+
+    def delete_tag(self, tag_name):
+        """Delete a tag definition.
+        """
+        self.branch.lock_write()
+        try:
+            d = self.get_tag_dict()
+            try:
+                del d[tag_name]
+            except KeyError:
+                raise errors.NoSuchTag(tag_name)
+            self._set_tag_dict(d)
+        finally:
+            self.branch.unlock()
+
+    def _set_tag_dict(self, new_dict):
+        """Replace all tag definitions
+
+        :param new_dict: Dictionary from tag name to target.
+        """
+        self.branch.lock_read()
+        try:
+            self.branch._transport.put_bytes('tags',
+                self._serialize_tag_dict(new_dict))
+        finally:
+            self.branch.unlock()
+
+    def _serialize_tag_dict(self, tag_dict):
+        td = dict((k.encode('utf-8'), v)
+                for k,v in tag_dict.items())
+        return bencode.bencode(td)
+
+    def _deserialize_tag_dict(self, tag_content):
+        """Convert the tag file into a dictionary of tags"""
+        # was a special case to make initialization easy, an empty definition
+        # is an empty dictionary
+        if tag_content == '':
+            return {}
+        try:
+            r = {}
+            for k, v in bencode.bdecode(tag_content).items():
+                r[k.decode('utf-8')] = v
+            return r
+        except ValueError, e:
+            raise ValueError("failed to deserialize tag dictionary %r: %s"
+                % (tag_content, e))
+
+    def merge_to(self, to_tags):
+        """Copy tags between repositories if necessary and possible.
+        
+        This method has common command-line behaviour about handling 
+        error cases.
+
+        All new definitions are copied across, except that tags that already
+        exist keep their existing definitions.
+
+        :param to_tags: Branch to receive these tags
+        :param just_warn: If the destination doesn't support tags and the 
+            source does have tags, just give a warning.  Otherwise, raise
+            TagsNotSupported (default).
+
+        :returns: A list of tags that conflicted, each of which is 
+            (tagname, source_target, dest_target).
+        """
+        if self.branch == to_tags.branch:
+            return
+        if not self.supports_tags():
+            # obviously nothing to copy
+            return
+        source_dict = self.get_tag_dict()
+        if not source_dict:
+            # no tags in the source, and we don't want to clobber anything
+            # that's in the destination
+            return
+        to_tags.branch.lock_write()
+        try:
+            dest_dict = to_tags.get_tag_dict()
+            result, conflicts = self._reconcile_tags(source_dict, dest_dict)
+            if result != dest_dict:
+                to_tags._set_tag_dict(result)
+        finally:
+            to_tags.branch.unlock()
+        return conflicts
+
+    def _reconcile_tags(self, source_dict, dest_dict):
+        """Do a two-way merge of two tag dictionaries.
+
+        only in source => source value
+        only in destination => destination value
+        same definitions => that
+        different definitions => keep destination value, give a warning
+
+        :returns: (result_dict,
+            [(conflicting_tag, source_target, dest_target)])
+        """
+        conflicts = []
+        result = dict(dest_dict) # copy
+        for name, target in source_dict.items():
+            if name not in result:
+                result[name] = target
+            elif result[name] == target:
+                pass
+            else:
+                conflicts.append((name, target, result[name]))
+        return result, conflicts
+
+
+def _merge_tags_if_possible(from_branch, to_branch):
+    from_branch.tags.merge_to(to_branch.tags)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2007-03-01 03:52:05 +0000
+++ bzrlib/tests/__init__.py	2007-03-05 03:43:56 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1794,6 +1794,7 @@
                    'bzrlib.tests.test_hashcache',
                    'bzrlib.tests.test_http',
                    'bzrlib.tests.test_http_response',
+                   'bzrlib.tests.test_https_ca_bundle',
                    'bzrlib.tests.test_identitymap',
                    'bzrlib.tests.test_ignores',
                    'bzrlib.tests.test_inv',
@@ -1836,6 +1837,7 @@
                    'bzrlib.tests.test_status',
                    'bzrlib.tests.test_store',
                    'bzrlib.tests.test_symbol_versioning',
+                   'bzrlib.tests.test_tag',
                    'bzrlib.tests.test_testament',
                    'bzrlib.tests.test_textfile',
                    'bzrlib.tests.test_textmerge',

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-11-18 22:38:16 +0000
+++ bzrlib/tests/blackbox/__init__.py	2007-01-16 13:21:38 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -87,6 +87,7 @@
                      'bzrlib.tests.blackbox.test_shared_repository',
                      'bzrlib.tests.blackbox.test_sign_my_commits',
                      'bzrlib.tests.blackbox.test_status',
+                     'bzrlib.tests.blackbox.test_tags',
                      'bzrlib.tests.blackbox.test_testament',
                      'bzrlib.tests.blackbox.test_too_much',
                      'bzrlib.tests.blackbox.test_uncommit',

=== modified file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	2007-03-01 08:50:03 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2007-03-05 03:43:56 +0000
@@ -108,3 +108,15 @@
         url = self.get_readonly_url() + '/README'
         out, err = self.run_bzr('cat', url)
         self.assertEqual('contents of README\n', out)
+
+    def test_cat_no_working_tree(self):
+        wt = self.make_branch_and_tree('.')
+        self.build_tree(['README'])
+        wt.add('README')
+        wt.commit('Making sure there is a basis_tree available')
+        wt.branch.bzrdir.destroy_workingtree()
+
+        url = self.get_readonly_url() + '/README'
+        out, err = self.run_bzr('cat', url)
+        self.assertEqual('contents of README\n', out)
+        

=== modified file 'bzrlib/tests/blackbox/test_init.py'
--- bzrlib/tests/blackbox/test_init.py	2007-02-13 19:22:47 +0000
+++ bzrlib/tests/blackbox/test_init.py	2007-02-17 07:11:47 +0000
@@ -24,6 +24,7 @@
     branch as _mod_branch,
     )
 from bzrlib.bzrdir import BzrDirMetaFormat1
+from bzrlib.tests import TestSkipped
 from bzrlib.tests.blackbox import ExternalBase
 from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
 from bzrlib.workingtree import WorkingTree
@@ -109,6 +110,15 @@
         self.run_bzr('init')
         self.assertFalse(os.path.exists('.bzrignore'))
 
+    def test_init_unicode(self):
+        # Make sure getcwd can handle unicode filenames
+        try:
+            os.mkdir(u'mu-\xb5')
+        except UnicodeError:
+            raise TestSkipped("Unable to create Unicode filename")
+        # try to init unicode dir
+        self.run_bzr('init', u'mu-\xb5')
+
 
 class TestSFTPInit(TestCaseWithSFTPServer):
 

=== modified file 'bzrlib/tests/blackbox/test_merge.py'
--- bzrlib/tests/blackbox/test_merge.py	2007-02-14 21:57:40 +0000
+++ bzrlib/tests/blackbox/test_merge.py	2007-02-23 05:39:35 +0000
@@ -267,7 +267,7 @@
         self.pullable_branch()
         os.chdir('a')
         (out, err) = self.run_bzr('merge', '--pull', '../b')
-        self.assertContainsRe(err, '1 revision\\(s\\) pulled')
+        self.assertContainsRe(err, 'Now on revision 2\\.')
         tree_a = WorkingTree.open('.')
         self.assertEqual([self.id2], tree_a.get_parent_ids())
 

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2007-02-25 11:49:16 +0000
@@ -228,7 +228,8 @@
         os.chdir(u'../' + dirname2)
         txt = bzr('pull')
 
-        self.assertEqual(u'Using saved location: %s/\n' % (pwd,), txt)
+        self.assertEqual(u'Using saved location: %s/\n'
+                'No revisions to pull.\n' % (pwd,), txt)
 
         os.chdir('../' + dirname1)
         open('a', 'ab').write('and yet more\n')

=== modified file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	2007-02-14 21:57:40 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2007-02-25 11:49:16 +0000
@@ -297,10 +297,11 @@
         bundle_file.close()
 
         os.chdir('../branch_b')
-        output = self.run_bzr('pull', '../bundle')
-        self.assertEqual('', output[0])
-        self.assertEqual(' M  a\nAll changes applied successfully.\n'
-                         '1 revision(s) pulled.\n', output[1])
+        out, err = self.run_bzr('pull', '../bundle')
+        self.assertEqual(out,
+                         'Now on revision 2.\n')
+        self.assertEqual(err,
+                ' M  a\nAll changes applied successfully.\n')
 
         self.assertEqualDiff(tree_a.branch.revision_history(),
                              tree_b.branch.revision_history())
@@ -313,6 +314,6 @@
                              testament_b.as_text())
 
         # it is legal to attempt to pull an already-merged bundle
-        output = self.run_bzr('pull', '../bundle')
-        self.assertEqual('', output[0])
-        self.assertEqual('0 revision(s) pulled.\n', output[1])
+        out, err = self.run_bzr('pull', '../bundle')
+        self.assertEqual(err, '')
+        self.assertEqual(out, 'No revisions to pull.\n')

=== modified file 'bzrlib/tests/blackbox/test_push.py'
--- bzrlib/tests/blackbox/test_push.py	2007-02-14 09:51:20 +0000
+++ bzrlib/tests/blackbox/test_push.py	2007-02-25 11:17:26 +0000
@@ -80,12 +80,14 @@
         # test implicit --remember after resolving previous failure
         uncommit(branch=branch_b, tree=tree_b)
         transport.delete('branch_b/c')
-        out = self.run_bzr('push')
+        out, err = self.run_bzr('push')
         path = branch_a.get_push_location()
-        self.assertEquals(('Using saved location: %s\n' 
-                           % (local_path_from_url(path),)
-                          , 'All changes applied successfully.\n'
-                            '1 revision(s) pushed.\n'), out)
+        self.assertEquals(out,
+                          'Using saved location: %s\n' 
+                          'Pushed up to revision 2.\n'
+                          % local_path_from_url(path))
+        self.assertEqual(err,
+                         'All changes applied successfully.\n')
         self.assertEqual(path,
                          branch_b.bzrdir.root_transport.base)
         # test explicit --remember
@@ -98,7 +100,7 @@
         b = self.make_branch('.')
         out, err = self.run_bzr('push', 'pushed-location')
         self.assertEqual('', out)
-        self.assertEqual('0 revision(s) pushed.\n', err)
+        self.assertEqual('Created new branch.\n', err)
         b2 = Branch.open('pushed-location')
         self.assertEndsWith(b2.base, 'pushed-location/')
 
@@ -114,7 +116,7 @@
         out, err = self.run_bzr('push', 'pushed-to')
         os.chdir('..')
         self.assertEqual('', out)
-        self.assertEqual('1 revision(s) pushed.\n', err)
+        self.assertEqual('Created new branch.\n', err)
 
     def test_push_only_pushes_history(self):
         # Knit branches should only push the history for the current revision.

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2007-02-07 15:38:35 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2007-02-27 22:08:46 +0000
@@ -329,7 +329,7 @@
         out, err = self.run_bzr_subprocess('rocks',
                         env_changes={'NON_EXISTANT_ENV_VAR':None},
                         universal_newlines=True)
-        self.assertEqual('it sure does!\n', out)
+        self.assertEqual('It sure does!\n', out)
         self.assertEqual('', err)
 
     def test_run_bzr_subprocess_working_dir(self):
@@ -449,7 +449,7 @@
 
     def test_run_bzr_error(self):
         out, err = self.run_bzr_error(['^$'], 'rocks', retcode=0)
-        self.assertEqual(out, 'it sure does!\n')
+        self.assertEqual(out, 'It sure does!\n')
 
         out, err = self.run_bzr_error(["bzr: ERROR: foobarbaz is not versioned"],
                                       'file-id', 'foobarbaz')

=== modified file 'bzrlib/tests/blackbox/test_serve.py'
--- bzrlib/tests/blackbox/test_serve.py	2006-10-16 01:50:48 +0000
+++ bzrlib/tests/blackbox/test_serve.py	2007-02-26 09:20:26 +0000
@@ -126,11 +126,6 @@
 
         self.assertServerFinishesCleanly(process)
 
-    def test_bzr_serve_no_args(self):
-        """'bzr serve' with no arguments or options should not traceback."""
-        out, err = self.run_bzr_error(
-            ['bzr serve requires one of --inet or --port'], 'serve')
-
     def test_bzr_connect_to_bzr_ssh(self):
         """User acceptance that get_transport of a bzr+ssh:// behaves correctly.
 

=== added file 'bzrlib/tests/blackbox/test_tags.py'
--- bzrlib/tests/blackbox/test_tags.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_tags.py	2007-02-26 05:36:12 +0000
@@ -0,0 +1,132 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for commands related to tags"""
+
+from bzrlib.branch import (
+    Branch,
+    BzrBranchExperimental,
+    )
+from bzrlib.bzrdir import BzrDir
+from bzrlib.tests import TestCaseWithTransport
+from bzrlib.repository import (
+    Repository,
+    )
+from bzrlib.repofmt.knitrepo import (
+    RepositoryFormatKnit2,
+    )
+from bzrlib.workingtree import WorkingTree
+
+
+class TestTagging(TestCaseWithTransport):
+
+    # as of 0.14, the default format doesn't do tags so we need to use a
+    # specific format
+    
+    def make_branch_and_tree(self, relpath):
+        control = BzrDir.create(relpath)
+        repo = RepositoryFormatKnit2().initialize(control)
+        BzrBranchExperimental.initialize(control)
+        return control.create_workingtree()
+
+    def test_tag_command_help(self):
+        out, err = self.run_bzr_captured(['help', 'tag'])
+        self.assertContainsRe(out, 'Create a tag')
+
+    def test_cannot_tag_range(self):
+        out, err = self.run_bzr('tag', '-r1..10', 'name', retcode=3)
+        self.assertContainsRe(err,
+            "Tags can only be placed on a single revision")
+
+    def test_tag_current_rev(self):
+        t = self.make_branch_and_tree('branch')
+        t.commit(allow_pointless=True, message='initial commit',
+            rev_id='first-revid')
+        # make a tag through the command line
+        out, err = self.run_bzr('tag', '-d', 'branch', 'NEWTAG')
+        self.assertContainsRe(out, 'Created tag NEWTAG.')
+        # tag should be observable through the api
+        self.assertEquals(t.branch.tags.get_tag_dict(),
+                dict(NEWTAG='first-revid'))
+        # can also create tags using -r
+        self.run_bzr('tag', '-d', 'branch', 'tag2', '-r1')
+        self.assertEquals(t.branch.tags.lookup_tag('tag2'), 'first-revid')
+        # can also delete an existing tag
+        out, err = self.run_bzr('tag', '--delete', '-d', 'branch', 'tag2')
+        # cannot replace an existing tag normally
+        out, err = self.run_bzr('tag', '-d', 'branch', 'NEWTAG', retcode=3)
+        self.assertContainsRe(err, 'Tag NEWTAG already exists\\.')
+        # ... but can if you use --force
+        out, err = self.run_bzr('tag', '-d', 'branch', 'NEWTAG', '--force')
+
+    def test_branch_push_pull_merge_copies_tags(self):
+        t = self.make_branch_and_tree('branch1')
+        t.commit(allow_pointless=True, message='initial commit',
+            rev_id='first-revid')
+        b1 = t.branch
+        b1.tags.set_tag('tag1', 'first-revid')
+        # branching copies the tag across
+        self.run_bzr('branch', 'branch1', 'branch2')
+        b2 = Branch.open('branch2')
+        self.assertEquals(b2.tags.lookup_tag('tag1'), 'first-revid')
+        # make a new tag and pull it
+        b1.tags.set_tag('tag2', 'twa')
+        self.run_bzr('pull', '-d', 'branch2', 'branch1')
+        self.assertEquals(b2.tags.lookup_tag('tag2'), 'twa')
+        # make a new tag and push it
+        b1.tags.set_tag('tag3', 'san')
+        self.run_bzr('push', '-d', 'branch1', 'branch2')
+        self.assertEquals(b2.tags.lookup_tag('tag3'), 'san')
+        # make a new tag and merge it
+        t.commit(allow_pointless=True, message='second commit',
+            rev_id='second-revid')
+        t2 = WorkingTree.open('branch2')
+        t2.commit(allow_pointless=True, message='commit in second')
+        b1.tags.set_tag('tag4', 'second-revid')
+        self.run_bzr('merge', '-d', 'branch2', 'branch1')
+        self.assertEquals(b2.tags.lookup_tag('tag4'), 'second-revid')
+
+    def test_list_tags(self):
+        t = self.make_branch_and_tree('branch1')
+        b1 = t.branch
+        tagname = u'\u30d0zaar'
+        b1.tags.set_tag(tagname, 'revid-1')
+        out, err = self.run_bzr('tags', '-d', 'branch1', encoding='utf-8')
+        self.assertEquals(err, '')
+        self.assertContainsRe(out,
+            u'^\u30d0zaar  *revid-1\n'.encode('utf-8'))
+
+    def test_conflicting_tags(self):
+        # setup two empty branches with different tags
+        t1 = self.make_branch_and_tree('one')
+        t2 = self.make_branch_and_tree('two')
+        b1 = t1.branch
+        b2 = t2.branch
+        tagname = u'\u30d0zaar'
+        b1.tags.set_tag(tagname, 'revid1')
+        b2.tags.set_tag(tagname, 'revid2')
+        # push should give a warning about the tags
+        out, err = self.run_bzr('push', '-d', 'one', 'two', encoding='utf-8')
+        self.assertContainsRe(out,
+                'Conflicting tags:\n.*' + tagname.encode('utf-8'))
+        # pull should give a warning about the tags
+        out, err = self.run_bzr('pull', '-d', 'one', 'two', encoding='utf-8')
+        self.assertContainsRe(out,
+                'Conflicting tags:\n.*' + tagname.encode('utf-8'))
+        # merge should give a warning about the tags -- not implemented yet
+        ## out, err = self.run_bzr('merge', '-d', 'one', 'two', encoding='utf-8')
+        ## self.assertContainsRe(out,
+        ##         'Conflicting tags:\n.*' + tagname.encode('utf-8'))

=== modified file 'bzrlib/tests/branch_implementations/__init__.py'
--- bzrlib/tests/branch_implementations/__init__.py	2007-02-06 02:33:42 +0000
+++ bzrlib/tests/branch_implementations/__init__.py	2007-02-15 01:23:29 +0000
@@ -1,6 +1,6 @@
-# Copyright (C) 2006 Canonical Ltd
+# Copyright (C) 2006, 2007 Canonical Ltd
 # Authors: Robert Collins <robert.collins@canonical.com>
-# -*- coding: utf-8 -*-
+#          and others
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -50,6 +50,7 @@
         'bzrlib.tests.branch_implementations.test_parent',
         'bzrlib.tests.branch_implementations.test_permissions',
         'bzrlib.tests.branch_implementations.test_pull',
+        'bzrlib.tests.branch_implementations.test_tags',
         'bzrlib.tests.branch_implementations.test_push',
         'bzrlib.tests.branch_implementations.test_uncommit',
         'bzrlib.tests.branch_implementations.test_update',

=== modified file 'bzrlib/tests/branch_implementations/test_pull.py'
--- bzrlib/tests/branch_implementations/test_pull.py	2007-02-06 02:33:42 +0000
+++ bzrlib/tests/branch_implementations/test_pull.py	2007-03-01 07:15:55 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005 Canonical Ltd
+# Copyright (C) 2004, 2005, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -80,6 +80,19 @@
         self.assertRaises(errors.BoundBranchConnectionFailure,
                 checkout.branch.pull, other.branch)
 
+    def test_pull_overwrite(self):
+        tree_a = self.make_branch_and_tree('tree_a')
+        tree_a.commit('message 1')
+        tree_b = tree_a.bzrdir.sprout('tree_b').open_workingtree()
+        tree_a.commit('message 2', rev_id='rev2a')
+        tree_b.commit('message 2', rev_id='rev2b')
+        self.assertRaises(errors.DivergedBranches, tree_a.pull, tree_b.branch)
+        tree_a.branch.pull(tree_a.branch, overwrite=True,
+                           stop_revision='rev2b')
+        self.assertEqual('rev2b', tree_a.branch.last_revision())
+        self.assertEqual(tree_b.branch.revision_history(),
+                         tree_a.branch.revision_history())
+
 
 class TestPullHook(TestCaseWithBranch):
 
@@ -87,22 +100,24 @@
         self.hook_calls = []
         TestCaseWithBranch.setUp(self)
 
-    def capture_post_pull_hook(self, source, local, master, old_revno,
-        old_revid, new_revno, new_revid):
+    def capture_post_pull_hook(self, result):
         """Capture post pull hook calls to self.hook_calls.
         
         The call is logged, as is some state of the two branches.
         """
-        if local:
-            local_locked = local.is_locked()
-            local_base = local.base
+        if result.local_branch:
+            local_locked = result.local_branch.is_locked()
+            local_base = result.local_branch.base
         else:
             local_locked = None
             local_base = None
         self.hook_calls.append(
-            ('post_pull', source, local_base, master.base, old_revno, old_revid,
-             new_revno, new_revid, source.is_locked(), local_locked,
-             master.is_locked()))
+            ('post_pull', result.source_branch, local_base,
+             result.master_branch.base, result.old_revno,
+             result.old_revid,
+             result.new_revno, result.new_revid,
+             result.source_branch.is_locked(), local_locked,
+             result.master_branch.is_locked()))
 
     def test_post_pull_empty_history(self):
         target = self.make_branch('target')

=== modified file 'bzrlib/tests/branch_implementations/test_push.py'
--- bzrlib/tests/branch_implementations/test_push.py	2007-02-12 22:58:48 +0000
+++ bzrlib/tests/branch_implementations/test_push.py	2007-03-01 04:15:28 +0000
@@ -36,8 +36,13 @@
         other.commit('my change', rev_id='M1', allow_pointless=True)
         mine.merge_from_branch(other.branch)
         mine.commit('merge my change', rev_id='P2')
-        mine.branch.push(other.branch)
+        result = mine.branch.push(other.branch)
         self.assertEqual(['P1', 'P2'], other.branch.revision_history())
+        # result object contains some structured data
+        self.assertEqual(result.old_revid, 'M1')
+        self.assertEqual(result.new_revid, 'P2')
+        # and it can be treated as an integer for compatibility
+        self.assertEqual(int(result), 0)
 
     def test_push_merged_indirect(self):
         # it should be possible to do a push from one branch into another
@@ -143,22 +148,24 @@
         self.hook_calls = []
         TestCaseWithBranch.setUp(self)
 
-    def capture_post_push_hook(self, source, local, master, old_revno,
-        old_revid, new_revno, new_revid):
+    def capture_post_push_hook(self, result):
         """Capture post push hook calls to self.hook_calls.
         
         The call is logged, as is some state of the two branches.
         """
-        if local:
-            local_locked = local.is_locked()
-            local_base = local.base
+        if result.local_branch:
+            local_locked = result.local_branch.is_locked()
+            local_base = result.local_branch.base
         else:
             local_locked = None
             local_base = None
         self.hook_calls.append(
-            ('post_push', source, local_base, master.base, old_revno, old_revid,
-             new_revno, new_revid, source.is_locked(), local_locked,
-             master.is_locked()))
+            ('post_push', result.source_branch, local_base,
+             result.master_branch.base,
+             result.old_revno, result.old_revid,
+             result.new_revno, result.new_revid,
+             result.source_branch.is_locked(), local_locked,
+             result.master_branch.is_locked()))
 
     def test_post_push_empty_history(self):
         target = self.make_branch('target')

=== added file 'bzrlib/tests/branch_implementations/test_tags.py'
--- bzrlib/tests/branch_implementations/test_tags.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/branch_implementations/test_tags.py	2007-02-26 05:29:05 +0000
@@ -0,0 +1,175 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tags stored within a branch
+
+The tags are actually in the Branch.tags namespace, but these are 
+1:1 with Branch implementations so can be tested from here.
+"""
+
+import os
+import re
+import sys
+
+import bzrlib
+from bzrlib import bzrdir, errors, repository
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
+from bzrlib.trace import mutter
+from bzrlib.workingtree import WorkingTree
+
+from bzrlib.tests.branch_implementations.test_branch \
+        import TestCaseWithBranch
+
+
+class TestBranchTags(TestCaseWithBranch):
+
+    def setUp(self):
+        # formats that don't support tags can skip the rest of these 
+        # tests...
+        fmt = self.branch_format
+        f = getattr(fmt, 'supports_tags')
+        if f is None:
+            raise TestSkipped("format %s doesn't declare whether it "
+                "supports tags, assuming not" % fmt)
+        if not f():
+            raise TestSkipped("format %s doesn't support tags" % fmt)
+        TestCaseWithBranch.setUp(self)
+
+    def test_tags_initially_empty(self):
+        b = self.make_branch('b')
+        tags = b.tags.get_tag_dict()
+        self.assertEqual(tags, {})
+
+    def test_make_and_lookup_tag(self):
+        b = self.make_branch('b')
+        b.tags.set_tag('tag-name', 'target-revid-1')
+        b.tags.set_tag('other-name', 'target-revid-2')
+        # then reopen the branch and see they're still there
+        b = Branch.open('b')
+        self.assertEqual(b.tags.get_tag_dict(),
+            {'tag-name': 'target-revid-1',
+             'other-name': 'target-revid-2',
+            })
+        # read one at a time
+        result = b.tags.lookup_tag('tag-name')
+        self.assertEqual(result, 'target-revid-1')
+        # and try has_tag
+        self.assertTrue(b.tags.has_tag('tag-name'))
+        self.assertFalse(b.tags.has_tag('imaginary'))
+
+    def test_no_such_tag(self):
+        b = self.make_branch('b')
+        try:
+            b.tags.lookup_tag('bosko')
+        except errors.NoSuchTag, e:
+            self.assertEquals(e.tag_name, 'bosko')
+            self.assertEquals(str(e), 'No such tag: bosko')
+        else:
+            self.fail("didn't get expected exception")
+
+    def test_merge_tags(self):
+        b1 = self.make_branch('b1')
+        b2 = self.make_branch('b2')
+        # if there are tags in the source and not the destination, then they
+        # just go across
+        b1.tags.set_tag('tagname', 'revid')
+        b1.tags.merge_to(b2.tags)
+        self.assertEquals(b2.tags.lookup_tag('tagname'), 'revid')
+        # if a tag is in the destination and not in the source, it is not
+        # removed when we merge them
+        b2.tags.set_tag('in-destination', 'revid')
+        result = b1.tags.merge_to(b2.tags)
+        self.assertEquals(result, [])
+        self.assertEquals(b2.tags.lookup_tag('in-destination'), 'revid')
+        # if there's a conflicting tag, it's reported -- the command line
+        # interface will say "these tags couldn't be copied"
+        b1.tags.set_tag('conflicts', 'revid-1')
+        b2.tags.set_tag('conflicts', 'revid-2')
+        result = b1.tags.merge_to(b2.tags)
+        self.assertEquals(result,
+            [('conflicts', 'revid-1', 'revid-2')])
+        # and it keeps the same value
+        self.assertEquals(b2.tags.lookup_tag('conflicts'), 'revid-2')
+
+
+    def test_unicode_tag(self):
+        b1 = self.make_branch('b')
+        tag_name = u'\u3070'
+        # in anticipation of the planned change to treating revision ids as
+        # just 8bit strings
+        revid = ('revid' + tag_name).encode('utf-8')
+        b1.tags.set_tag(tag_name, revid)
+        self.assertEquals(b1.tags.lookup_tag(tag_name), revid)
+
+    def test_delete_tag(self):
+        b = self.make_branch('b')
+        tag_name = u'\N{GREEK SMALL LETTER ALPHA}'
+        revid = ('revid' + tag_name).encode('utf-8')
+        b.tags.set_tag(tag_name, revid)
+        # now try to delete it
+        b.tags.delete_tag(tag_name)
+        # now you can't look it up
+        self.assertRaises(errors.NoSuchTag,
+            b.tags.lookup_tag, tag_name)
+        # and it's not in the dictionary
+        self.assertEquals(b.tags.get_tag_dict(), {})
+        # and you can't remove it a second time
+        self.assertRaises(errors.NoSuchTag,
+            b.tags.delete_tag, tag_name)
+        # or remove a tag that never existed
+        self.assertRaises(errors.NoSuchTag,
+            b.tags.delete_tag, tag_name + '2')
+
+    def test_merge_empty_tags(self):
+        # you can merge tags between two instances, since neither have tags
+        b1 = self.make_branch('b1')
+        b2 = self.make_branch('b2')
+        b1.tags.merge_to(b2.tags)
+
+
+class TestUnsupportedTags(TestCaseWithBranch):
+    """Formats that don't support tags should give reasonable errors."""
+
+    def setUp(self):
+        fmt = self.branch_format
+        supported = getattr(fmt, 'supports_tags')
+        if supported is None:
+            warn("Format %s doesn't declare whether it supports tags or not"
+                 % fmt)
+            raise TestSkipped('No tag support at all')
+        if supported():
+            raise TestSkipped("Format %s declares that tags are supported"
+                              % fmt)
+            # it's covered by TestBranchTags
+        TestCaseWithBranch.setUp(self)
+    
+    def test_tag_methods_raise(self):
+        b = self.make_branch('b')
+        self.assertRaises(errors.TagsNotSupported,
+            b.tags.set_tag, 'foo', 'bar')
+        self.assertRaises(errors.TagsNotSupported,
+            b.tags.lookup_tag, 'foo')
+        self.assertRaises(errors.TagsNotSupported,
+            b.tags.set_tag, 'foo', 'bar')
+        self.assertRaises(errors.TagsNotSupported,
+            b.tags.delete_tag, 'foo')
+
+    def test_merge_empty_tags(self):
+        # you can merge tags between two instances, since neither have tags
+        b1 = self.make_branch('b1')
+        b2 = self.make_branch('b2')
+        b1.tags.merge_to(b2.tags)

=== modified file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	2007-03-01 07:27:36 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2007-03-05 03:43:56 +0000
@@ -48,9 +48,6 @@
 
 class TestCaseWithRepository(TestCaseWithBzrDir):
 
-    def setUp(self):
-        super(TestCaseWithRepository, self).setUp()
-
     def make_branch(self, relpath, format=None):
         repo = self.make_repository(relpath, format=None)
         return repo.bzrdir.create_branch()

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2007-02-15 14:08:23 +0000
+++ bzrlib/tests/test_branch.py	2007-02-23 05:55:44 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,9 +30,15 @@
     errors,
     urlutils,
     )
-import bzrlib.branch
-from bzrlib.branch import (BzrBranch5, 
-                           BzrBranchFormat5)
+from bzrlib.branch import (
+    Branch,
+    BranchHooks,
+    BranchFormat,
+    BranchReferenceFormat,
+    BzrBranch5,
+    BzrBranchFormat5,
+    PullResult,
+    )
 from bzrlib.bzrdir import (BzrDirMetaFormat1, BzrDirMeta1, 
                            BzrDir, BzrDirFormat)
 from bzrlib.errors import (NotBranchError,
@@ -47,10 +53,10 @@
 class TestDefaultFormat(TestCase):
 
     def test_get_set_default_format(self):
-        old_format = bzrlib.branch.BranchFormat.get_default_format()
+        old_format = BranchFormat.get_default_format()
         # default is 5
-        self.assertTrue(isinstance(old_format, bzrlib.branch.BzrBranchFormat5))
-        bzrlib.branch.BranchFormat.set_default_format(SampleBranchFormat())
+        self.assertTrue(isinstance(old_format, BzrBranchFormat5))
+        BranchFormat.set_default_format(SampleBranchFormat())
         try:
             # the default branch format is used by the meta dir format
             # which is not the default bzrdir format at this point
@@ -58,8 +64,8 @@
             result = dir.create_branch()
             self.assertEqual(result, 'A branch')
         finally:
-            bzrlib.branch.BranchFormat.set_default_format(old_format)
-        self.assertEqual(old_format, bzrlib.branch.BranchFormat.get_default_format())
+            BranchFormat.set_default_format(old_format)
+        self.assertEqual(old_format, BranchFormat.get_default_format())
 
 
 class TestBranchFormat5(TestCaseWithTransport):
@@ -99,7 +105,7 @@
     # recursive section - that is, it appends the branch name.
 
 
-class SampleBranchFormat(bzrlib.branch.BranchFormat):
+class SampleBranchFormat(BranchFormat):
     """A sample format
 
     this format is initializable, unsupported to aid in testing the 
@@ -135,21 +141,21 @@
             dir = format._matchingbzrdir.initialize(url)
             dir.create_repository()
             format.initialize(dir)
-            found_format = bzrlib.branch.BranchFormat.find_format(dir)
+            found_format = BranchFormat.find_format(dir)
             self.failUnless(isinstance(found_format, format.__class__))
-        check_format(bzrlib.branch.BzrBranchFormat5(), "bar")
+        check_format(BzrBranchFormat5(), "bar")
         
     def test_find_format_not_branch(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         self.assertRaises(NotBranchError,
-                          bzrlib.branch.BranchFormat.find_format,
+                          BranchFormat.find_format,
                           dir)
 
     def test_find_format_unknown_format(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         SampleBranchFormat().initialize(dir)
         self.assertRaises(UnknownFormatError,
-                          bzrlib.branch.BranchFormat.find_format,
+                          BranchFormat.find_format,
                           dir)
 
     def test_register_unregister_format(self):
@@ -159,14 +165,14 @@
         # make a branch
         format.initialize(dir)
         # register a format for it.
-        bzrlib.branch.BranchFormat.register_format(format)
+        BranchFormat.register_format(format)
         # which branch.Open will refuse (not supported)
-        self.assertRaises(UnsupportedFormatError, bzrlib.branch.Branch.open, self.get_url())
+        self.assertRaises(UnsupportedFormatError, Branch.open, self.get_url())
         self.make_branch_and_tree('foo')
         # but open_downlevel will work
         self.assertEqual(format.open(dir), bzrdir.BzrDir.open(self.get_url()).open_branch(unsupported=True))
         # unregister the format
-        bzrlib.branch.BranchFormat.unregister_format(format)
+        BranchFormat.unregister_format(format)
         self.make_branch_and_tree('bar')
 
     def test_checkout_format(self):
@@ -263,7 +269,7 @@
         target_branch = dir.create_branch()
         t.mkdir('branch')
         branch_dir = bzrdirformat.initialize(self.get_url('branch'))
-        made_branch = bzrlib.branch.BranchReferenceFormat().initialize(branch_dir, target_branch)
+        made_branch = BranchReferenceFormat().initialize(branch_dir, target_branch)
         self.assertEqual(made_branch.base, target_branch.base)
         opened_branch = branch_dir.open_branch()
         self.assertEqual(opened_branch.base, target_branch.base)
@@ -273,7 +279,7 @@
 
     def test_constructor(self):
         """Check that creating a BranchHooks instance has the right defaults."""
-        hooks = bzrlib.branch.BranchHooks()
+        hooks = BranchHooks()
         self.assertTrue("set_rh" in hooks, "set_rh not in %s" % hooks)
         self.assertTrue("post_push" in hooks, "post_push not in %s" % hooks)
         self.assertTrue("post_commit" in hooks, "post_commit not in %s" % hooks)
@@ -283,15 +289,29 @@
     def test_installed_hooks_are_BranchHooks(self):
         """The installed hooks object should be a BranchHooks."""
         # the installed hooks are saved in self._preserved_hooks.
-        self.assertIsInstance(self._preserved_hooks, bzrlib.branch.BranchHooks)
+        self.assertIsInstance(self._preserved_hooks, BranchHooks)
 
     def test_install_hook_raises_unknown_hook(self):
         """install_hook should raise UnknownHook if a hook is unknown."""
-        hooks = bzrlib.branch.BranchHooks()
+        hooks = BranchHooks()
         self.assertRaises(UnknownHook, hooks.install_hook, 'silly', None)
 
     def test_install_hook_appends_known_hook(self):
         """install_hook should append the callable for known hooks."""
-        hooks = bzrlib.branch.BranchHooks()
+        hooks = BranchHooks()
         hooks.install_hook('set_rh', None)
         self.assertEqual(hooks['set_rh'], [None])
+
+
+class TestPullResult(TestCase):
+
+    def test_pull_result_to_int(self):
+        # to support old code, the pull result can be used as an int
+        r = PullResult()
+        r.old_revno = 10
+        r.new_revno = 20
+        # this usage of results is not recommended for new code (because it
+        # doesn't describe very well what happened), but for api stability
+        # it's still supported
+        a = "%d revisions pulled" % r
+        self.assertEqual(a, "10 revisions pulled")

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2007-02-15 14:08:23 +0000
+++ bzrlib/tests/test_bzrdir.py	2007-02-21 05:34:56 +0000
@@ -142,6 +142,7 @@
         finally:
             bzrdir.format_registry.set_default_repository(old_default)
 
+
 class SampleBranch(bzrlib.branch.Branch):
     """A dummy branch for guess what, dummy use."""
 

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2007-02-08 17:01:20 +0000
+++ bzrlib/tests/test_http.py	2007-02-28 21:40:44 +0000
@@ -126,10 +126,23 @@
         self.port = None
 
 
+class TestWithTransport_pycurl(object):
+    """Test case to inherit from if pycurl is present"""
+
+    def _get_pycurl_maybe(self):
+        try:
+            from bzrlib.transport.http._pycurl import PyCurlTransport
+            return PyCurlTransport
+        except errors.DependencyNotPresent:
+            raise TestSkipped('pycurl not present')
+
+    _transport = property(_get_pycurl_maybe)
+
+
 class TestHttpUrls(TestCase):
 
-    # FIXME: Some of these tests should be done for both
-    # implementations
+    # TODO: This should be moved to authorization tests once they
+    # are written.
 
     def test_url_parsing(self):
         f = FakeManager()
@@ -142,46 +155,102 @@
         self.assertEquals([None, 'www.bazaar-vcs.org', 'user', 'pass'],
                           f.credentials[0])
 
+
+class TestHttpTransportUrls(object):
+    """Test the http urls.
+
+    This MUST be used by daughter classes that also inherit from
+    TestCase.
+
+    We can't inherit directly from TestCase or the
+    test framework will try to create an instance which cannot
+    run, its implementation being incomplete.
+    """
+
     def test_abs_url(self):
         """Construction of absolute http URLs"""
-        t = HttpTransport_urllib('http://bazaar-vcs.org/bzr/bzr.dev/')
+        t = self._transport('http://bazaar-vcs.org/bzr/bzr.dev/')
         eq = self.assertEqualDiff
-        eq(t.abspath('.'),
-           'http://bazaar-vcs.org/bzr/bzr.dev')
-        eq(t.abspath('foo/bar'),
-           'http://bazaar-vcs.org/bzr/bzr.dev/foo/bar')
-        eq(t.abspath('.bzr'),
-           'http://bazaar-vcs.org/bzr/bzr.dev/.bzr')
+        eq(t.abspath('.'), 'http://bazaar-vcs.org/bzr/bzr.dev')
+        eq(t.abspath('foo/bar'), 'http://bazaar-vcs.org/bzr/bzr.dev/foo/bar')
+        eq(t.abspath('.bzr'), 'http://bazaar-vcs.org/bzr/bzr.dev/.bzr')
         eq(t.abspath('.bzr/1//2/./3'),
            'http://bazaar-vcs.org/bzr/bzr.dev/.bzr/1/2/3')
 
     def test_invalid_http_urls(self):
         """Trap invalid construction of urls"""
-        t = HttpTransport_urllib('http://bazaar-vcs.org/bzr/bzr.dev/')
-        self.assertRaises(ValueError,
-            t.abspath,
-            '.bzr/')
-        t = HttpTransport_urllib('http://http://bazaar-vcs.org/bzr/bzr.dev/')
-        self.assertRaises(errors.InvalidURL, t.has, 'foo/bar')
+        t = self._transport('http://bazaar-vcs.org/bzr/bzr.dev/')
+        self.assertRaises(ValueError, t.abspath, '.bzr/')
+        t = self._transport('http://http://bazaar-vcs.org/bzr/bzr.dev/')
+        self.assertRaises((errors.InvalidURL, errors.ConnectionError),
+                          t.has, 'foo/bar')
 
     def test_http_root_urls(self):
         """Construction of URLs from server root"""
-        t = HttpTransport_urllib('http://bzr.ozlabs.org/')
+        t = self._transport('http://bzr.ozlabs.org/')
         eq = self.assertEqualDiff
         eq(t.abspath('.bzr/tree-version'),
            'http://bzr.ozlabs.org/.bzr/tree-version')
 
     def test_http_impl_urls(self):
         """There are servers which ask for particular clients to connect"""
-        server = HttpServer_PyCurl()
+        server = self._server()
         try:
             server.setUp()
             url = server.get_url()
-            self.assertTrue(url.startswith('http+pycurl://'))
+            self.assertTrue(url.startswith('%s://' % self._qualified_prefix))
         finally:
             server.tearDown()
 
 
+class TestHttpUrls_urllib(TestHttpTransportUrls, TestCase):
+    """Test http urls with urllib"""
+
+    _transport = HttpTransport_urllib
+    _server = HttpServer_urllib
+    _qualified_prefix = 'http+urllib'
+
+
+class TestHttpUrls_pycurl(TestWithTransport_pycurl, TestHttpTransportUrls,
+                          TestCase):
+    """Test http urls with pycurl"""
+
+    _server = HttpServer_PyCurl
+    _qualified_prefix = 'http+pycurl'
+
+    # TODO: This should really be moved into another pycurl
+    # specific test. When https tests will be implemented, take
+    # this one into account.
+    def test_pycurl_without_https_support(self):
+        """Test that pycurl without SSL do not fail with a traceback.
+
+        For the purpose of the test, we force pycurl to ignore
+        https by supplying a fake version_info that do not
+        support it.
+        """
+        try:
+            import pycurl
+        except ImportError:
+            raise TestSkipped('pycurl not present')
+        # Now that we have pycurl imported, we can fake its version_info
+        # This was taken from a windows pycurl without SSL
+        # (thanks to bialix)
+        pycurl.version_info = lambda : (2,
+                                        '7.13.2',
+                                        462082,
+                                        'i386-pc-win32',
+                                        2576,
+                                        None,
+                                        0,
+                                        None,
+                                        ('ftp', 'gopher', 'telnet',
+                                         'dict', 'ldap', 'http', 'file'),
+                                        None,
+                                        0,
+                                        None)
+        self.assertRaises(errors.DependencyNotPresent, self._transport,
+                          'https://launchpad.net')
+
 class TestHttpConnections(object):
     """Test the http connections.
 
@@ -247,19 +316,6 @@
             socket.setdefaulttimeout(default_timeout)
 
 
-class TestWithTransport_pycurl(object):
-    """Test case to inherit from if pycurl is present"""
-
-    def _get_pycurl_maybe(self):
-        try:
-            from bzrlib.transport.http._pycurl import PyCurlTransport
-            return PyCurlTransport
-        except errors.DependencyNotPresent:
-            raise TestSkipped('pycurl not present')
-
-    _transport = property(_get_pycurl_maybe)
-
-
 class TestHttpConnections_urllib(TestHttpConnections, TestCaseWithWebserver):
     """Test http connections with urllib"""
 

=== added file 'bzrlib/tests/test_https_ca_bundle.py'
--- bzrlib/tests/test_https_ca_bundle.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_https_ca_bundle.py	2007-02-26 20:28:08 +0000
@@ -0,0 +1,70 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Testing of bzrlib.transport.http.ca_bundle module"""
+
+import os
+import sys
+
+import bzrlib
+from bzrlib import osutils
+from bzrlib.tests import (
+    TestCaseInTempDir,
+    TestSkipped,
+    )
+from bzrlib.transport.http import ca_bundle
+
+
+class TestGetCAPath(TestCaseInTempDir):
+
+    def setUp(self):
+        TestCaseInTempDir.setUp(self)
+        new_env = {
+            'CURL_CA_BUNDLE': None,
+            'PATH': None,
+            }
+        self._old_env = {}
+        self.addCleanup(self._restore)
+        for name, value in new_env.iteritems():
+            self._old_env[name] = osutils.set_or_unset_env(name, None)
+
+    def _restore(self):
+        for name, value in self._old_env.iteritems():
+            osutils.set_or_unset_env(name, value)
+
+    def _make_file(self, in_dir='.'):
+        fname = os.path.join(in_dir, 'curl-ca-bundle.crt')
+        f = file(fname,'w')
+        f.write('spam')
+        f.close()
+
+    def test_found_nothing(self):
+        self.assertEqual('', ca_bundle.get_ca_path(use_cache=False))
+
+    def test_env_var(self):
+        osutils.set_or_unset_env('CURL_CA_BUNDLE', 'foo.bar')
+        self._make_file()
+        self.assertEqual('foo.bar', ca_bundle.get_ca_path(use_cache=False))
+
+    def test_in_path(self):
+        if sys.platform != 'win32':
+            raise TestSkipped('Searching in PATH implemented only for win32')
+        os.mkdir('foo')
+        in_dir = os.path.join(os.getcwd(), 'foo')
+        self._make_file(in_dir=in_dir)
+        osutils.set_or_unset_env('PATH', in_dir)
+        shouldbe = os.path.join(in_dir, 'curl-ca-bundle.crt')
+        self.assertEqual(shouldbe, ca_bundle.get_ca_path(use_cache=False))

=== modified file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_msgeditor.py	2007-02-04 21:58:50 +0000
@@ -80,9 +80,12 @@
         self.assertEqual(True, bzrlib.msgeditor._run_editor(''),
                          'Unable to run dummy fake editor')
 
-    def test_edit_commit_message(self):
-        working_tree = self.make_uncommitted_tree()
-        # make fake editor
+    def make_fake_editor(self):
+        """Set up environment so that an editor will be a known script.
+
+        Sets up BZR_EDITOR so that if an editor is spawned it will run a
+        script that just adds a known message to the start of the file.
+        """
         f = file('fed.py', 'wb')
         f.write('#!%s\n' % sys.executable)
         f.write("""\
@@ -112,6 +115,10 @@
             os.chmod('fed.py', 0755)
             os.environ['BZR_EDITOR'] = './fed.py'
 
+    def test_edit_commit_message(self):
+        working_tree = self.make_uncommitted_tree()
+        self.make_fake_editor()
+
         mutter('edit_commit_message without infotext')
         self.assertEqual('test message from fed\n',
                          bzrlib.msgeditor.edit_commit_message(''))
@@ -120,6 +127,16 @@
         self.assertEqual('test message from fed\n',
                          bzrlib.msgeditor.edit_commit_message(u'\u1234'))
 
+    def test_start_message(self):
+        self.make_uncommitted_tree()
+        self.make_fake_editor()
+        self.assertEqual('test message from fed\nstart message\n',
+                         bzrlib.msgeditor.edit_commit_message('',
+                                              start_message='start message\n'))
+        self.assertEqual('test message from fed\n',
+                         bzrlib.msgeditor.edit_commit_message('',
+                                              start_message=''))
+
     def test_deleted_commit_message(self):
         working_tree = self.make_uncommitted_tree()
 

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2007-02-13 07:28:10 +0000
+++ bzrlib/tests/test_repository.py	2007-02-15 01:23:29 +0000
@@ -336,7 +336,6 @@
         self.assertTrue(S_ISDIR(t.stat('knits').st_mode))
         self.check_knits(t)
 
-
 class InterString(repository.InterRepository):
     """An inter-repository optimised code path for strings.
 
@@ -474,3 +473,4 @@
         tree.commit("Another dull commit", rev_id='dull2')
         revision_tree = tree.branch.repository.revision_tree('dull2')
         self.assertEqual('dull', revision_tree.inventory.root.revision)
+

=== modified file 'bzrlib/tests/test_revisionnamespaces.py'
--- bzrlib/tests/test_revisionnamespaces.py	2006-10-16 10:03:21 +0000
+++ bzrlib/tests/test_revisionnamespaces.py	2007-02-20 07:50:00 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2004, 2005, 2006 Canonical Ltd
+# Copyright (C) 2004, 2005, 2006, 2007 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,11 +19,18 @@
 import time
 
 from bzrlib import (
+    branch,
+    bzrdir,
     errors,
+    repository,
     )
 from bzrlib.builtins import merge
 from bzrlib.tests import TestCase, TestCaseWithTransport
-from bzrlib.revisionspec import RevisionSpec, RevisionSpec_revno
+from bzrlib.revisionspec import (
+    RevisionSpec,
+    RevisionSpec_revno,
+    RevisionSpec_tag,
+    )
 
 
 def spec_in_history(spec, branch):
@@ -335,9 +342,28 @@
 
 class TestRevisionSpec_tag(TestRevisionSpec):
     
-    def test_invalid(self):
-        self.assertInvalid('tag:foo', extra='\ntag: namespace registered,'
-                                            ' but not implemented')
+    def make_branch_and_tree(self, relpath):
+        # override format as the default one may not support tags
+        control = bzrdir.BzrDir.create(relpath)
+        control.create_repository()
+        branch.BzrBranchExperimental.initialize(control)
+        return control.create_workingtree()
+
+    def test_from_string_tag(self):
+        spec = RevisionSpec.from_string('tag:bzr-0.14')
+        self.assertIsInstance(spec, RevisionSpec_tag)
+        self.assertEqual(spec.spec, 'bzr-0.14')
+
+    def test_lookup_tag(self):
+        self.tree.branch.tags.set_tag('bzr-0.14', 'r1')
+        self.assertInHistoryIs(1, 'r1', 'tag:bzr-0.14')
+
+    def test_failed_lookup(self):
+        # tags that don't exist give a specific message: arguably we should
+        # just give InvalidRevisionSpec but I think this is more helpful
+        self.assertRaises(errors.NoSuchTag,
+            self.get_in_history,
+            'tag:some-random-tag')
 
 
 class TestRevisionSpec_date(TestRevisionSpec):

=== added file 'bzrlib/tests/test_tag.py'
--- bzrlib/tests/test_tag.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_tag.py	2007-02-22 04:52:06 +0000
@@ -0,0 +1,81 @@
+# Copyright (C) 2006, 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib import (
+    branch,
+    bzrdir,
+    errors,
+    tag,
+    )
+from bzrlib.tag import (
+    BasicTags,
+    _merge_tags_if_possible,
+    )
+from bzrlib.tests import (
+    TestCase,
+    TestCaseInTempDir,
+    )
+
+
+class TestTagSerialization(TestCase):
+
+    def test_tag_serialization(self):
+        """Test the precise representation of tag dicts."""
+        # Don't change this after we commit to this format, as it checks 
+        # that the format is stable and compatible across releases.
+        #
+        # This release stores them in bencode as a dictionary from name to
+        # target.
+        store = BasicTags(branch=None)
+        td = dict(stable='stable-revid', boring='boring-revid')
+        packed = store._serialize_tag_dict(td)
+        expected = r'd6:boring12:boring-revid6:stable12:stable-revide'
+        self.assertEqualDiff(packed, expected)
+        self.assertEqual(store._deserialize_tag_dict(packed), td)
+
+
+class TestTagMerging(TestCaseInTempDir):
+
+    def make_knit_branch(self, relpath):
+        old_bdf = bzrdir.format_registry.make_bzrdir('knit')
+        return bzrdir.BzrDir.create_branch_convenience(relpath, format=old_bdf)
+
+    def make_branch_supporting_tags(self, relpath):
+        new_bdf = bzrdir.format_registry.make_bzrdir('experimental-branch6')
+        return bzrdir.BzrDir.create_branch_convenience(relpath, format=new_bdf)
+
+    def test_merge_not_possible(self):
+        # test merging between branches which do and don't support tags
+        old_branch = self.make_knit_branch('old')
+        new_branch = self.make_branch_supporting_tags('new')
+        # just to make sure this test is valid
+        self.assertFalse(old_branch.supports_tags(),
+            "%s is expected to not support tags but does" % old_branch)
+        self.assertTrue(new_branch.supports_tags(),
+            "%s is expected to support tags but does not" % new_branch)
+        # there are no tags in the old one, and we can merge from it into the
+        # new one
+        old_branch.tags.merge_to(new_branch.tags)
+        # we couldn't merge tags from the new branch to the old one, but as
+        # there are not any yet this isn't a problem
+        new_branch.tags.merge_to(old_branch.tags)
+        # but if there is a tag in the new one, we get a warning when trying
+        # to move it back
+        new_branch.tags.set_tag(u'\u2040tag', 'revid')
+        old_branch.tags.merge_to(new_branch.tags)
+        self.assertRaises(errors.TagsNotSupported,
+            new_branch.tags.merge_to, old_branch.tags)

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2007-03-01 06:19:44 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2007-03-05 03:43:56 +0000
@@ -562,11 +562,29 @@
         self.assertEqual(master_tree.branch.revision_history(),
             tree.branch.revision_history())
 
-    def test_merge_modified(self):
+    def test_merge_modified_detects_corruption(self):
+        # FIXME: This doesn't really test that it works; also this is not
+        # implementation-independent. mbp 20070226
         tree = self.make_branch_and_tree('master')
         tree._control_files.put('merge-hashes', StringIO('asdfasdf'))
         self.assertRaises(errors.MergeModifiedFormatError, tree.merge_modified)
 
+    def test_merge_modified(self):
+        # merge_modified stores a map from file id to hash
+        tree = self.make_branch_and_tree('tree')
+        d = {'file-id': osutils.sha_string('hello')}
+        self.build_tree_contents([('tree/somefile', 'hello')])
+        tree.lock_write()
+        try:
+            tree.add(['somefile'], ['file-id'])
+            tree.set_merge_modified(d)
+            mm = tree.merge_modified()
+            self.assertEquals(mm, d)
+        finally:
+            tree.unlock()
+        mm = tree.merge_modified()
+        self.assertEquals(mm, d)
+
     def test_conflicts(self):
         from bzrlib.tests.test_conflicts import example_conflicts
         tree = self.make_branch_and_tree('master')

=== modified file 'bzrlib/transform.py'
--- bzrlib/transform.py	2007-03-05 01:15:25 +0000
+++ bzrlib/transform.py	2007-03-05 03:43:56 +0000
@@ -1387,7 +1387,8 @@
             child_pb.finished()
         conflicts = cook_conflicts(raw_conflicts, tt)
         if change_reporter:
-            change_reporter = delta.ChangeReporter()
+            change_reporter = delta.ChangeReporter(
+                unversioned_filter=working_tree.is_ignored)
             delta.report_changes(tt._iter_changes(), change_reporter)
         for conflict in conflicts:
             warning(conflict)

=== modified file 'bzrlib/transport/http/_pycurl.py'
--- bzrlib/transport/http/_pycurl.py	2006-12-13 05:07:19 +0000
+++ bzrlib/transport/http/_pycurl.py	2007-03-01 03:08:30 +0000
@@ -25,6 +25,7 @@
 
 import os
 from cStringIO import StringIO
+import sys
 
 from bzrlib import (
     errors,
@@ -37,6 +38,7 @@
 from bzrlib.trace import mutter
 from bzrlib.transport import register_urlparse_netloc_protocol
 from bzrlib.transport.http import (
+    ca_bundle,
     _extract_headers,
     HttpTransportBase,
     _pycurl_errors,
@@ -79,6 +81,13 @@
 
     def __init__(self, base, from_transport=None):
         super(PyCurlTransport, self).__init__(base)
+        if base.startswith('https'):
+            # Check availability of https into pycurl supported
+            # protocols
+            supported = pycurl.version_info()[8]
+            if 'https' not in supported:
+                raise DependencyNotPresent('pycurl', 'no https support')
+        self.cabundle = ca_bundle.get_ca_path()
         if from_transport is not None:
             self._curl = from_transport._curl
         else:
@@ -230,6 +239,8 @@
         ua_str = 'bzr/%s (pycurl)' % (bzrlib.__version__,)
         curl.setopt(pycurl.USERAGENT, ua_str)
         curl.setopt(pycurl.FOLLOWLOCATION, 1) # follow redirect responses
+        if self.cabundle:
+            curl.setopt(pycurl.CAINFO, self.cabundle)
 
     def _curl_perform(self, curl, more_headers=[]):
         """Perform curl operation and translate exceptions."""

=== added file 'bzrlib/transport/http/ca_bundle.py'
--- bzrlib/transport/http/ca_bundle.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/http/ca_bundle.py	2007-02-27 06:57:37 +0000
@@ -0,0 +1,77 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Auto-detect of CA bundle for SSL connections"""
+
+import os
+import sys
+from bzrlib.trace import mutter
+
+
+_ca_path = None
+
+
+def get_ca_path(use_cache=True):
+    """Return location of CA bundle"""
+    global _ca_path
+
+    if _ca_path is not None and use_cache:
+        return _ca_path
+
+    # Find CA bundle for SSL
+    # Reimplementation in Python the magic of curl command line tool
+    # from "Details on Server SSL Certificates"
+    # http://curl.haxx.se/docs/sslcerts.html
+    #
+    # 4. If you're using the curl command line tool, you can specify your own CA
+    #    cert path by setting the environment variable CURL_CA_BUNDLE to the path
+    #    of your choice.
+    #
+    #    If you're using the curl command line tool on Windows, curl will search
+    #    for a CA cert file named "curl-ca-bundle.crt" in these directories and in
+    #    this order:
+    #      1. application's directory
+    #      2. current working directory
+    #      3. Windows System directory (e.g. C:\windows\system32)
+    #      4. Windows Directory (e.g. C:\windows)
+    #      5. all directories along %PATH%
+    #
+    # NOTES:
+    #   bialix: Windows directories usually listed in PATH env variable
+    #   j-a-meinel: bzr should not look in current working dir
+
+    path = os.environ.get('CURL_CA_BUNDLE')
+    if not path and sys.platform == 'win32':
+        dirs = [os.path.realpath(os.path.dirname(sys.argv[0]))]     # app dir
+        paths = os.environ.get('PATH')
+        if paths:
+            # don't include the cwd in the search
+            paths = [i for i in paths.split(os.pathsep) if i not in ('', '.')]
+            dirs.extend(paths)
+        for d in dirs:
+            fname = os.path.join(d, "curl-ca-bundle.crt")
+            if os.path.isfile(fname):
+                path = fname
+                break
+    if path:
+        mutter('using CA bundle: %r', path)
+    else:
+        path = ''
+
+    if use_cache:
+        _ca_path = path
+
+    return path

=== modified file 'bzrlib/transport/smart.py'
--- bzrlib/transport/smart.py	2007-02-15 06:15:43 +0000
+++ bzrlib/transport/smart.py	2007-02-27 02:20:50 +0000
@@ -223,6 +223,10 @@
 del scheme
 
 
+# Port 4155 is the default port for bzr://, registered with IANA.
+BZR_DEFAULT_PORT = 4155
+
+
 def _recv_tuple(from_file):
     req_line = from_file.readline()
     return _decode_tuple(req_line)
@@ -1731,10 +1735,14 @@
     def __init__(self, url):
         _scheme, _username, _password, _host, _port, _path = \
             transport.split_url(url)
-        try:
-            _port = int(_port)
-        except (ValueError, TypeError), e:
-            raise errors.InvalidURL(path=url, extra="invalid port %s" % _port)
+        if _port is None:
+            _port = BZR_DEFAULT_PORT
+        else:
+            try:
+                _port = int(_port)
+            except (ValueError, TypeError), e:
+                raise errors.InvalidURL(
+                    path=url, extra="invalid port %s" % _port)
         medium = SmartTCPClientMedium(_host, _port)
         super(SmartTCPTransport, self).__init__(url, medium=medium)
 

=== added file 'bzrlib/util/bencode.py'
--- bzrlib/util/bencode.py	1970-01-01 00:00:00 +0000
+++ bzrlib/util/bencode.py	2007-02-20 04:47:54 +0000
@@ -0,0 +1,328 @@
+# bencode structured encoding
+#
+# Written by Petru Paler
+#
+# Permission is hereby granted, free of charge, to any person
+# obtaining a copy of this software and associated documentation files
+# (the "Software"), to deal in the Software without restriction,
+# including without limitation the rights to use, copy, modify, merge,
+# publish, distribute, sublicense, and/or sell copies of the Software,
+# and to permit persons to whom the Software is furnished to do so,
+# subject to the following conditions:
+# 
+# The above copyright notice and this permission notice shall be
+# included in all copies or substantial portions of the Software.
+# 
+# The Software is provided "AS IS", without warranty of any kind,
+# express or implied, including but not limited to the warranties of
+# merchantability,  fitness for a particular purpose and
+# noninfringement. In no event shall the  authors or copyright holders
+# be liable for any claim, damages or other liability, whether in an
+# action of contract, tort or otherwise, arising from, out of or in
+# connection with the Software or the use or other dealings in the
+# Software.
+
+def decode_int(x, f):
+    f += 1
+    newf = x.index('e', f)
+    try:
+        n = int(x[f:newf])
+    except (OverflowError, ValueError):
+        n = long(x[f:newf])
+    if x[f] == '-':
+        if x[f + 1] == '0':
+            raise ValueError
+    elif x[f] == '0' and newf != f+1:
+        raise ValueError
+    return (n, newf+1)
+
+def decode_string(x, f):
+    colon = x.index(':', f)
+    try:
+        n = int(x[f:colon])
+    except (OverflowError, ValueError):
+        n = long(x[f:colon])
+    if x[f] == '0' and colon != f+1:
+        raise ValueError
+    colon += 1
+    return (x[colon:colon+n], colon+n)
+
+def decode_list(x, f):
+    r, f = [], f+1
+    while x[f] != 'e':
+        v, f = decode_func[x[f]](x, f)
+        r.append(v)
+    return (r, f + 1)
+
+def decode_dict(x, f):
+    r, f = {}, f+1
+    lastkey = None
+    while x[f] != 'e':
+        k, f = decode_string(x, f)
+        if lastkey >= k:
+            raise ValueError
+        lastkey = k
+        r[k], f = decode_func[x[f]](x, f)
+    return (r, f + 1)
+
+decode_func = {}
+decode_func['l'] = decode_list
+decode_func['d'] = decode_dict
+decode_func['i'] = decode_int
+decode_func['0'] = decode_string
+decode_func['1'] = decode_string
+decode_func['2'] = decode_string
+decode_func['3'] = decode_string
+decode_func['4'] = decode_string
+decode_func['5'] = decode_string
+decode_func['6'] = decode_string
+decode_func['7'] = decode_string
+decode_func['8'] = decode_string
+decode_func['9'] = decode_string
+
+def bdecode(x):
+    try:
+        r, l = decode_func[x[0]](x, 0)
+    except (IndexError, KeyError):
+        raise ValueError
+    if l != len(x):
+        raise ValueError
+    return r
+
+def test_bdecode():
+    try:
+        bdecode('0:0:')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('ie')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('i341foo382e')
+        assert 0
+    except ValueError:
+        pass
+    assert bdecode('i4e') == 4L
+    assert bdecode('i0e') == 0L
+    assert bdecode('i123456789e') == 123456789L
+    assert bdecode('i-10e') == -10L
+    try:
+        bdecode('i-0e')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('i123')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('i6easd')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('35208734823ljdahflajhdf')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('2:abfdjslhfld')
+        assert 0
+    except ValueError:
+        pass
+    assert bdecode('0:') == ''
+    assert bdecode('3:abc') == 'abc'
+    assert bdecode('10:1234567890') == '1234567890'
+    try:
+        bdecode('02:xy')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('l')
+        assert 0
+    except ValueError:
+        pass
+    assert bdecode('le') == []
+    try:
+        bdecode('leanfdldjfh')
+        assert 0
+    except ValueError:
+        pass
+    assert bdecode('l0:0:0:e') == ['', '', '']
+    try:
+        bdecode('relwjhrlewjh')
+        assert 0
+    except ValueError:
+        pass
+    assert bdecode('li1ei2ei3ee') == [1, 2, 3]
+    assert bdecode('l3:asd2:xye') == ['asd', 'xy']
+    assert bdecode('ll5:Alice3:Bobeli2ei3eee') == [['Alice', 'Bob'], [2, 3]]
+    try:
+        bdecode('d')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('defoobar')
+        assert 0
+    except ValueError:
+        pass
+    assert bdecode('de') == {}
+    assert bdecode('d3:agei25e4:eyes4:bluee') == {'age': 25, 'eyes': 'blue'}
+    assert bdecode('d8:spam.mp3d6:author5:Alice6:lengthi100000eee') == {'spam.mp3': {'author': 'Alice', 'length': 100000}}
+    try:
+        bdecode('d3:fooe')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('di1e0:e')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('d1:b0:1:a0:e')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('d1:a0:1:a0:e')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('i03e')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('l01:ae')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('9999:x')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('l0:')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('d0:0:')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('d0:')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('00:')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('l-3:e')
+        assert 0
+    except ValueError:
+        pass
+    try:
+        bdecode('i-03e')
+        assert 0
+    except ValueError:
+        pass
+    bdecode('d0:i3ee')
+
+from types import StringType, IntType, LongType, DictType, ListType, TupleType
+
+class Bencached(object):
+    __slots__ = ['bencoded']
+
+    def __init__(self, s):
+        self.bencoded = s
+
+def encode_bencached(x,r):
+    r.append(x.bencoded)
+
+def encode_int(x, r):
+    r.extend(('i', str(x), 'e'))
+
+def encode_string(x, r):
+    r.extend((str(len(x)), ':', x))
+
+def encode_list(x, r):
+    r.append('l')
+    for i in x:
+        encode_func[type(i)](i, r)
+    r.append('e')
+
+def encode_dict(x,r):
+    r.append('d')
+    ilist = x.items()
+    ilist.sort()
+    for k, v in ilist:
+        r.extend((str(len(k)), ':', k))
+        encode_func[type(v)](v, r)
+    r.append('e')
+
+encode_func = {}
+encode_func[type(Bencached(0))] = encode_bencached
+encode_func[IntType] = encode_int
+encode_func[LongType] = encode_int
+encode_func[StringType] = encode_string
+encode_func[ListType] = encode_list
+encode_func[TupleType] = encode_list
+encode_func[DictType] = encode_dict
+
+try:
+    from types import BooleanType
+    encode_func[BooleanType] = encode_int
+except ImportError:
+    pass
+
+def bencode(x):
+    r = []
+    encode_func[type(x)](x, r)
+    return ''.join(r)
+
+def test_bencode():
+    assert bencode(4) == 'i4e'
+    assert bencode(0) == 'i0e'
+    assert bencode(-10) == 'i-10e'
+    assert bencode(12345678901234567890L) == 'i12345678901234567890e'
+    assert bencode('') == '0:'
+    assert bencode('abc') == '3:abc'
+    assert bencode('1234567890') == '10:1234567890'
+    assert bencode([]) == 'le'
+    assert bencode([1, 2, 3]) == 'li1ei2ei3ee'
+    assert bencode([['Alice', 'Bob'], [2, 3]]) == 'll5:Alice3:Bobeli2ei3eee'
+    assert bencode({}) == 'de'
+    assert bencode({'age': 25, 'eyes': 'blue'}) == 'd3:agei25e4:eyes4:bluee'
+    assert bencode({'spam.mp3': {'author': 'Alice', 'length': 100000}}) == 'd8:spam.mp3d6:author5:Alice6:lengthi100000eee'
+    assert bencode(Bencached(bencode(3))) == 'i3e'
+    try:
+        bencode({1: 'foo'})
+    except TypeError:
+        return
+    assert 0
+
+try:
+    import psyco
+    psyco.bind(bdecode)
+    psyco.bind(bencode)
+except ImportError:
+    pass

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-03-02 01:06:12 +0000
+++ bzrlib/workingtree.py	2007-03-05 03:43:56 +0000
@@ -785,8 +785,8 @@
                 yield Stanza(file_id=file_id.decode('utf8'), hash=hash)
         self._put_rio('merge-hashes', iter_stanzas(), MERGE_MODIFIED_HEADER_1)
 
-    @needs_tree_write_lock
     def _put_rio(self, filename, stanzas, header):
+        self._must_be_locked()
         my_file = rio_file(stanzas, header)
         self._control_files.put(filename, my_file)
 
@@ -837,6 +837,15 @@
 
     @needs_read_lock
     def merge_modified(self):
+        """Return a dictionary of files modified by a merge.
+
+        The list is initialized by WorkingTree.set_merge_modified, which is 
+        typically called after we make some automatic updates to the tree
+        because of a merge.
+
+        This returns a map of file_id->sha1, containing only files which are
+        still in the working inventory and have that text hash.
+        """
         try:
             hashfile = self._control_files.get('merge-hashes')
         except errors.NoSuchFile:
@@ -851,9 +860,9 @@
             file_id = s.get("file_id")
             if file_id not in self.inventory:
                 continue
-            hash = s.get("hash")
-            if hash == self.get_file_sha1(file_id):
-                merge_hashes[file_id] = hash
+            text_hash = s.get("hash")
+            if text_hash == self.get_file_sha1(file_id):
+                merge_hashes[file_id] = text_hash
         return merge_hashes
 
     @needs_write_lock
@@ -1526,6 +1535,10 @@
     def is_locked(self):
         return self._control_files.is_locked()
 
+    def _must_be_locked(self):
+        if not self.is_locked():
+            raise errors.ObjectNotLocked(self)
+
     def lock_read(self):
         """See Branch.lock_read, and WorkingTree.unlock."""
         self.branch.lock_read()

=== modified file 'doc/centralized_workflow.txt'
--- doc/centralized_workflow.txt	2006-10-31 20:40:59 +0000
+++ doc/centralized_workflow.txt	2007-03-02 12:40:45 +0000
@@ -13,7 +13,7 @@
 manner. Bazaar_ is designed to be very flexible and allows several
 different workflows, from fully decentralized to mostly centralized.  The
 workflow used here is meant to ease a new user into more advanced usage of
-Bazaar_, and allows them to work in a mix of centralized and decentralized
+Bazaar_, and allow them to work in a mix of centralized and decentralized
 operations.
 
 In general, this document is meant for users coming from a background of
@@ -56,11 +56,11 @@
 needs so that you do not need to copy around all of this history
 information whenever you create a new branch.
 
-The best way to do this is to create a `Shared Repository`_. In general,
-branches will share their storage if they exist in a subdirectory of a
-`Shared Repository`_.  So let's setup a `Shared Repository`_ in our home
-directory, thus all branches we create underneath will share their history
-storage.
+The best way to do this is to create a `Shared Repository`_. In
+general, branches will share their storage if they exist in a
+subdirectory of a `Shared Repository`_.  So let's set up a `Shared
+Repository`_ in our home directory, thus all branches we create
+underneath will share their history storage.
 
 ::
 
@@ -87,11 +87,14 @@
   % bzr init-repo --no-trees sftp://centralhost/srv/bzr/
 
 You can think of this step as similar to setting up a new cvsroot, or
-subversion repository.
-
-
-Versioning an existing project
-==============================
+subversion repository.  The {{{--no-trees}}} option tells bzr to not
+populate the directory with a working tree.  This is appropriate,
+since no one will be making changes directly in the branches within
+the central repository.
+
+
+Migrating an existing project to Bazaar
+=======================================
 
 Now that we have a repository, let's create a versioned project. Most of
 the time, you will already have some code that you are working with,
@@ -112,8 +115,8 @@
 ----------------------------------------
 
 So first, we want to create a branch in our remote Repository, where we
-will want to host the project.  Let's assume we have a project named
-"sigil" that we want to start versioning.
+want to host the project.  Let's assume we have a project named
+"sigil" that we want to put under version control.
 
 ::
 
@@ -141,16 +144,16 @@
   % bzr commit -m "Initial import of Sigil"
 
 
-In the example above, we created an empty branch (the ``/sigil`` branch)
-on ``centralhost``, and then checkout out this empty branch onto our
-workstation to add files from our existing project.
-There are many ways to setup your working directory, but the steps above
+In the previous section, we created an empty branch (the ``/sigil``
+branch) on ``centralhost``, and then checkout out this empty branch
+onto our workstation to add files from our existing project.  There
+are many ways to set up your working directory, but the steps above
 make it easy to handle working with feature/bugfix branches. And one
 of the strong points of Bazaar_ is how well it works with branches.
 
 At this point, because you have a 'checkout' of the remote branch, any
-commits you make in ``~/work/sigil/dev/`` will automatically be saved both locally,
-and on ``centralhost``.
+commits you make in ``~/work/sigil/dev/`` will automatically be saved
+both locally, and on ``centralhost``.
 
 
 Developer N: Getting a working copy of the project
@@ -170,7 +173,7 @@
 the checkouts will be out of date with the current version.
 At commit time, Bazaar_ will inform the user of this and prevent them from
 committing. To get up to date, use ``bzr update`` to update the
-tree with the remote changes. This may require resolving conflicts, if the
+tree with the remote changes. This may require resolving conflicts if the
 same files have been modified.
 
 
@@ -211,12 +214,12 @@
 not interrupt people who are working on other parts of the code.  Because
 we have a checkout, any commits made in the ``~/work/sigil/doodle-fixes/``
 will also show up on ``centralhost``. [#nestedbranches]_ It is also
-possible to have 2 developers collaborate on one of these branches, just
+possible to have two developers collaborate on one of these branches, just
 like they would have collaborated on the ``dev`` branch. [#cbranch]_
 
 .. [#nestedbranches] It may look odd to have a branch in a subdirectory of
    another branch. This is just fine, and you can think of it as a
-   heirarchial namespace. Where the nested branch is derived from the
+   hierarchical namespace where the nested branch is derived from the
    outer branch.
 
 .. [#cbranch] When using lots of independent branches, having to retype
@@ -226,7 +229,11 @@
    Which is designed to take a base branch, create a new public branch,
    and create a checkout of that branch, all with much less typing.
    Configuring ``cbranch`` is outside the scope of this document, but the
-   final commands look like ``bzr cbranch dev my-feature-branch``
+   final commands are similar to:
+
+::
+
+   % bzr cbranch dev my-feature-branch
 
 .. _bzrtools: https://launchpad.net/products/bzrtools
 
@@ -275,7 +282,8 @@
                sftp://centralhost/srv/bzr/sigil/user-b
 
 This gives each developer their own branch to work on. And, they can
-easily create a new feature branch for themselves with just[#cbranch]_::
+easily create a new feature branch for themselves with just [#cbranch]_
+::
 
   % bzr branch sftp://centralhost/srv/bzr/sigil/user-a \
                sftp://centralhost/srv/bzr/sigil/user-a/feature 
@@ -289,20 +297,21 @@
 Shared Repository
 -----------------
 
-Bazaar_ has the concept of a "Shared Repository". This is similar to the
-concept of other RCS's repository. Such as in Subversion, where you have a
-remote repository, which is where all of the history is stored, and
-locally you don't have any history information, only a checkout of the
-working tree files. Note that "Shared" in this context means shared
-between branches. It *may* be shared between people, but standalone
-branches can also be shared between people.
+Bazaar_ has the concept of a "Shared Repository". This is similar to
+the traditional concept of a repository in other RCSs like CVS and
+Subversion. For example, in Subversion you have a remote repository,
+which is where all of the history is stored, and locally you don't
+have any history information, only a checkout of the working tree
+files. Note that "Shared" in this context means shared between
+branches. It *may* be shared between people, but standalone branches
+can also be shared between people.
 
 In Bazaar_ terms, a "Shared Repository" is a location where multiple
-branches can **share** their revision history information. Because Bazaar_
-wants to support decentralized workflows, it is possible for every branch
-to maintain its own revision history information. But this is often
-inefficient, since related branches share history, and they might as well
-share the storage as well.
+branches can **share** their revision history information. In order to
+support decentralized workflows, it is possible for every branch to
+store its own revision history information. But this is often
+inefficient, since related branches share history, and they might as
+well share the storage as well.
 
 
 .. 

=== modified file 'doc/configuration.txt'
--- doc/configuration.txt	2007-02-13 22:34:57 +0000
+++ doc/configuration.txt	2007-03-02 12:40:45 +0000
@@ -6,10 +6,10 @@
 
 Location of configuration file
 ==============================
-Each user gets a pair of configurations files in $HOME/.bazaar. The first
-one, named bazaar.conf, includes default configuration options. The other
-file, locations.conf, contains configuration information for specific
-branch locations.
+Each user gets a pair of configurations files in ``$HOME/.bazaar``. The first
+one, named ``bazaar.conf``, includes default configuration options. The other
+file, ``locations.conf``, contains configuration information for specific
+branch locations.  These files are sometimes referred to as ``ini files``.
 
 General Format
 ==============
@@ -25,18 +25,18 @@
 Section Headers
 ---------------
 A section header is a word enclosed in brackets that starts at the begining
-of a line, typical section headers look like this::
+of a line.  A typical section header looks like this::
 
     [DEFAULT]
 
-The only valid section header for bazaar.conf is [DEFAULT], which is case
-senstive. The default section provides for setting variables which can be
-overridden with the branch config file.
+The only valid section header for bazaar.conf is [DEFAULT], which is
+case sensitive. The default section provides for setting variables
+which can be overridden with the branch config file.
 
-For locations.conf, the variables from the section with the longest matching
-section header are used to the exclusion of other potentially valid section
-headers. A section header uses the path for the branch as the section
-header. Some examples include::
+For ``locations.conf``, the variables from the section with the
+longest matching section header are used to the exclusion of other
+potentially valid section headers. A section header uses the path for
+the branch as the section header. Some examples include::
 
     [http://mybranches.isp.com/~jdoe/branchdir]
     [/home/jdoe/branches/]
@@ -47,8 +47,7 @@
 -----------------
 
 A section variable resides within a section. A section variable contains a
-variable name, an equals sign and a value and generally takes the following
-form::
+variable name, an equals sign and a value.  For example::
 
     email            = John Doe <jdoe@isp.com>
     check_signatures = require
@@ -87,12 +86,12 @@
 The main configuration file, bazaar.conf
 ----------------------------------------
 
-The main configuration file, $HOME/.bazaar/bazaar.conf, only allows one
-section called '''[DEFAULT]'''. This default section contains the default
+The main configuration file, ``$HOME/.bazaar/bazaar.conf``, only allows one
+section called ``[DEFAULT]``. This default section contains the default
 configuration options for all branches. The default section can be
-overriden by providing a branch specific section in locations.conf.
+overriden by providing a branch-specific section in ``locations.conf``.
 
-A typical bazaar.conf section often looks like the following::
+A typical ``bazaar.conf`` section often looks like the following::
 
     [DEFAULT]
     email             = John Doe <jdoe@isp.com>
@@ -100,11 +99,11 @@
     check_signatures  = check-available
     create_signatures = when-required
 
-$HOME/.bazaar/locations.conf allows one to specify overriding settings for a
+``$HOME/.bazaar/locations.conf`` allows one to specify overriding settings for a
 specific branch. The format is almost identical to the default section in
 bazaar.conf with one significant change: The section header, instead of
 saying default, will be the path to a branch that you wish to override a
-value for. The ? and * wildcards are supported::
+value for. The '?' and '*' wildcards are supported::
 
     [/home/jdoe/branches/nethack]
     email = Nethack Admin <nethack@nethack.com>
@@ -116,7 +115,7 @@
     [http://bazaar-vcs.org/bzr/*]
     check_signatures  = require
 
-Common Variable options
+Common Variable Options
 =======================
 
 email
@@ -130,14 +129,14 @@
 ------
 The path of the editor that you wish to use if *bzr commit* is run without
 a commit log message. This setting is trumped by the environment variable
-$BZR_EDITOR, and overrides $VISUAL and $EDITOR.
+``$BZR_EDITOR``, and overrides ``$VISUAL`` and ``$EDITOR``.
 
 check_signatures
 ----------------
 Defines the behavior for signatures.
 
 require
-    the gnupg signature for revisions must be present and must be valid
+    The gnupg signature for revisions must be present and must be valid.
 
 ignore
     Do not check gnupg signatures of revisions. 
@@ -145,39 +144,39 @@
 check-available
     (default) If gnupg signatures for revisions are present, check them.
     Bazaar will fail if it finds a bad signature, but will not fail if
-    no signature is present
+    no signature is present.
 
 create_signatures
 -----------------
-Defines the behaviour of signing revisions. Has three possible values:
-always, never and when-requied.
+Defines the behaviour of signing revisions.
 
 always
-    sign every new revision that is committed
+    Sign every new revision that is committed.
 
 when-required
     (default) Sign newly committed revisions only when the branch requires
-    signed revisions
+    signed revisions.
 
 never
-    Refuse to sign newly committed revisions, even if the branch requires signatures
+    Refuse to sign newly committed revisions, even if the branch
+    requires signatures.
 
 recurse
 -------
-Only useful in locations.conf. Defines whether or not the configuration for
-this section applies to subdirectories:
+Only useful in ``locations.conf``. Defines whether or not the
+configuration for this section applies to subdirectories:
 
 true
-    (default) This section applies to subdirectories as well
+    (default) This section applies to subdirectories as well.
 
 false
     This section only applies to the branch at this directory and not
-    branches 
+    branches below it. 
 
 gpg_signing_command
 -------------------
 (Default: "gpg"). Which program should be used to sign and check revisions.
-example::
+For example::
 
     gpg_signing_command = /usr/bin/gnpg
 
@@ -186,34 +185,33 @@
 ================
 
 These options apply only to branches that use the "experimental-branch6"
-format.  They are usually set in .bzr/branch/branch.conf automatically, but
-may be manually set in locations.conf or bazaar.conf
+format.  They are usually set in ``.bzr/branch/branch.conf`` automatically, but
+may be manually set in ``locations.conf`` or ``bazaar.conf``.
 
 append_revisions_only
 ---------------------
-If set to "True", then revisions can only be appended to the log, not removed.
-A branch with this setting enabled can only pull from another branch if the
-other branch's log is a longer version of its own.
-
-This is normally set by ``bzr init --append-revisions-only``.
+If set to "True" then revisions can only be appended to the log, not
+removed.  A branch with this setting enabled can only pull from
+another branch if the other branch's log is a longer version of its
+own.  This is normally set by ``bzr init --append-revisions-only``.
 
 parent_location
 ---------------
 If present, the location of the default branch for pull or merge.
-
-This is normally set by ``pull --remember`` or ``merge --remember``
+This option is normally set by ``pull --remember`` or ``merge
+--remember``
 
 push_location
 -------------
-If present, the location of the default branch for push.  This is normally
-set by ``push --remember``.
+If present, the location of the default branch for push.  This option
+is normally set by ``push --remember``.
 
 bound_location
 --------------
-The location that commits should go to when acting as a checkout.  (Normally
-set by ``bind``)
+The location that commits should go to when acting as a checkout.
+This option is normally set by ``bind``.
 
 bound
 -----
 If set to "True", the branch should act as a checkout, and push each commit to
-the bound_location.  (Normally set by ``bind``/``unbind``.)
+the bound_location.  This option is normally set by ``bind``/``unbind``.

=== modified file 'doc/http_smart_server.txt'
--- doc/http_smart_server.txt	2006-12-18 15:42:15 +0000
+++ doc/http_smart_server.txt	2007-03-02 12:40:45 +0000
@@ -5,8 +5,8 @@
 **This feature is EXPERIMENTAL and is NOT SECURE.  It will allow access to
 arbitrary files on your server.**
 
-This document describes one way to setup a Bazaar HTTP smart server, using
-Apache 2.0 and FastCGI or mod_python.
+This document describes one way to set up a Bazaar HTTP smart server,
+using Apache 2.0 and FastCGI or mod_python.
 
 Example
 =======

=== modified file 'doc/index.txt'
--- doc/index.txt	2006-10-05 10:23:20 +0000
+++ doc/index.txt	2007-03-02 12:40:45 +0000
@@ -41,7 +41,7 @@
 * `Using aliases <using_aliases.htm>`_
   (wiki page: http://bazaar-vcs.org/CommandAliases)
 
-    How to setup and use command aliases.
+    How to set up and use command aliases.
 
 * `Running a Bazaar server <server.htm>`_
   

=== modified file 'doc/plugins.txt'
--- doc/plugins.txt	2006-07-30 06:52:39 +0000
+++ doc/plugins.txt	2007-03-02 12:40:45 +0000
@@ -7,17 +7,18 @@
 What is a Plugin
 ================
 A plugin is an external component for Bazaar that is typically made by
-third parties. A plugin is capable of augmenting Bazaar by adding or
-replacing new or current functionality in Bazaar. Plugins often work as
-a way for developers to test new features for Bazaar prior to inclusion
-in the official codebase. Plugins can do a variety of things, including
+third parties. A plugin is capable of augmenting Bazaar by adding new
+functionality.  A plugin can also change current Bazaar behavior by
+replacing current functionality. Plugins often work as a way for
+developers to test new features for Bazaar prior to inclusion in the
+official codebase. Plugins can do a variety of things, including
 overriding commands, adding new commands, providing additional network
-transports, customizing log output. The sky is the limit for the
+transports, or customizing log output. The sky is the limit for the
 customization that can be done through plugins.
 
 Where to find Plugins 
 =====================
-We keep our list of plugins on the http://bazaar-vcs.org/PluginRegistry page.
+We keep our list of plugins on the http://bazaar-vcs.org/BzrPlugins page.
 
 Writing a plugin
 ================
@@ -26,22 +27,22 @@
 but most plugins supply new commands.
 
 To create a command, make a new object that derives from
-`bzrlib.commands.Command`, and name it cmd_foo, where foo is the name of
+``bzrlib.commands.Command``, and name it ``cmd_foo``, where foo is the name of
 your command.  If you create a command whose name contains an underscore,
 it will appear in the UI with the underscore turned into a hyphen.  For
-example, "cmd_baz_import" will appear as "baz-import".  For examples of how
-to write commands, please see builtins.py.
+example, `cmd_baz_import` will appear as `baz-import`.  For examples of how
+to write commands, please see ``builtins.py``.
 
 Once you've created a command you must register the command with
-`bzrlib.commands.register_command(cmd_foo)`.  You must register the command
-when your file is imported, otherwise bzr will not see it.
+``bzrlib.commands.register_command(cmd_foo)``.  You must register the
+command when your file is imported, otherwise bzr will not see it.
 
-Bzr will scan **bzrlib/plugins** and **~/.bazaar/plugins** for plugins by
-default.  You can override this with **BZR_PLUGIN_PATH**.  Plugins may be
-either modules or packages.  If your plugin is a single file, you can
-structure it as a module.  If it has multiple files, or if you want to
-distribute it as a bzr branch, you should structure it as a package, i.e. a
-directory with an **__init__.py** file.
+Bzr will scan ``bzrlib/plugins`` and ``~/.bazaar/plugins`` for plugins
+by default.  You can override this with ``BZR_PLUGIN_PATH``.  Plugins
+may be either modules or packages.  If your plugin is a single file,
+you can structure it as a module.  If it has multiple files, or if you
+want to distribute it as a bzr branch, you should structure it as a
+package, i.e. a directory with an ``__init__.py`` file.
 
 Please feel free to contribute your plugin to BzrTools, if you think it
 would be useful to other people.
@@ -49,18 +50,18 @@
 How to Install a plugin 
 =======================
 Installing a plugin is very easy! One can either install a plugin
-systemwide or on a user by user basis. Both methods involve create a
-"plugins" directory. Within this directory one can place plugins in
-subdirectories. For example, "plugins/bzrtools/".
+system-wide or on a per user basis. Both methods involve creating a
+``plugins`` directory. Within this directory one can place plugins in
+subdirectories. For example, ``plugins/bzrtools/``.
 
-Two locations are currently checked; the bzrlib/plugins directory
-(typically found in /usr/lib/python2.4/site-packages/bzrlib/plugins/) and
-$HOME/.bazaar/plugins/.
+Two locations are currently checked:  the bzrlib/plugins directory
+(typically found in ``/usr/lib/python2.4/site-packages/bzrlib/plugins/``) and
+``$HOME/.bazaar/plugins/``.
 
 One can additionally override the home plugins by setting the environment
-variable BZR_PLUGIN_PATH to a directory that contains plugins. The
-installation of a plugin can be checked by running **bzr plugins** at
-any time. New commands can be seen by running **bzr help commands**.
+variable ``BZR_PLUGIN_PATH`` to a directory that contains plugins. The
+installation of a plugin can be checked by running ``bzr plugins`` at
+any time. New commands can be seen by running ``bzr help commands``.
 
 Plugins work particularly well with Bazaar branches. For example, to
 install the bzrtools plugins for your main user account, one can perform

=== modified file 'doc/server.txt'
--- doc/server.txt	2006-09-18 09:58:40 +0000
+++ doc/server.txt	2007-03-02 12:40:45 +0000
@@ -18,22 +18,23 @@
 
 Bazaar supports writing over FTP, SFTP and via a plugin over HTTP-WebDAV.
 
-High performance server
+High-performance server
 =======================
 
 **In development**
 
-The high performance server is currently in development.  The version of Bazaar
+The high-performance server is currently in development.  The version of Bazaar
 that accompanies this documentation is able to use the servers underlying
-protocol as a dumb server.  This is the first stage high performance server
+protocol as a dumb server.  This is the first stage high-performance server
 functionality to be delivered.
 
-The current high performance server provides read only access by default for
-security. To enable read-write access, run it with --allow-writes. When using
-the SSH access method, bzr automatically runs with with the --allow-writes
-option.
+To maintain the highest security possible, the current
+high-performance server provides read-only access by default.  To
+enable read-write access, run it with ``--allow-writes``. When using
+the SSH access method, bzr automatically runs with with the
+``--allow-writes`` option.
 
-It can be configured to run in three different ways:
+The server can be configured to run in three different ways:
 
 SSH
 ---
@@ -42,35 +43,36 @@
 
     BZR_REMOTE_PATH=~/bin/bzr bzr log bzr+ssh://host/path/to/branch
 
-The `BZR_REMOTE_PATH` environment variable adjusts how `bzr` will be invoked on
-the remote system.  By default, just `bzr` will be invoked, which requires the
-`bzr` executable to be on the default search path.
+The ``BZR_REMOTE_PATH`` environment variable adjusts how `bzr` will be
+invoked on the remote system.  By default, just `bzr` will be invoked,
+which requires the `bzr` executable to be on the default search path.
 
-The `bzr+ssh://` URL scheme only supports absolute paths from the root of the
-filesystem.  Future versions are expected to support `~` in the same way as
-`sftp://` URLs.
+The ``bzr+ssh://`` URL scheme only supports absolute paths from the
+root of the filesystem.  Future versions are expected to support ``~``
+in the same way as ``sftp://`` URLs.
 
 inetd
 -----
 
-This example shows how to run `bzr` with a dedicated user `bzruser` for a shared
-repository in `/srv/bzr/repo` which has a branch at `/srv/bzr/repo/branchname`
+This example shows how to run `bzr` with a dedicated user `bzruser`
+for a shared repository in ``/srv/bzr/repo`` which has a branch at
+``/srv/bzr/repo/branchname``.
 
 Running a Bazaar server from inetd requires an inetd.conf entry::
 
     1234  stream  tcp  nowait  bzruser  /usr/bin/bzr serve --inet --directory=/srv/bzr/repo
 
 When running client commands, the URL you supply is a `bzr://` URL relative to
-the `--directory` option given in inetd.conf::
+the ``--directory`` option given in inetd.conf::
 
     bzr log bzr://host:1234/branchname
 
 Dedicated
 ---------
 
-This mode has the same path and URL behaviour as the inetd mode.  To run as a
-specific, you should use `su` or login as that user.  This example runs bzr
-serve on `localhost` port `1234`.
+This mode has the same path and URL behaviour as the inetd mode.  To
+run as a specific user, you should use ``su`` or login as that user.
+This example runs ``bzr serve`` on `localhost` port `1234`.
 
 server::
 

=== modified file 'doc/setting_up_email.txt'
--- doc/setting_up_email.txt	2006-07-30 06:52:39 +0000
+++ doc/setting_up_email.txt	2007-03-02 17:52:32 +0000
@@ -2,38 +2,44 @@
 Setting up email
 ================
 
-The various ways that exist to tell Bazaar your email address for
+Description of the various ways to specify to Bazaar your email address for
 commits.
 
 Why set up an email address with Bazaar?
 ========================================
-Bazaar stores the claimed email address in revisions when they're
-created, so that people can tell who commited which revisions.  Of course,
-people can change the email address claimed, so you have to trust the
-people involved.  Additionally, the email address in a revision gives
-others a way to contact the author of a revision for credit and/or blame.
-:)
+Bazaar stores the specified email address in revisions when they're
+created so that people can tell who commited which revisions.  The
+email addresses are not verified, therefore they could be bogus, so
+you have to trust the people involved in your project.  Additionally,
+the email address in a revision gives others a way to contact the
+author of a revision for credit and/or blame.  :)
 
-How to setup your email address 
-===============================
+How to set up your email address 
+================================
 Bazaar will try to guess an email address based on your username and the
 hostname if none is set.  This will probably not be what you want, so three
-ways exist to tell Bazaar what email to use
+ways exist to tell Bazaar what email to use:
 
- * You can set your email in the default ini file,
- * You can set emails per-branch in the branch specific ini file,
- * Or your email can be set by an environmental variable.
+You can set your email in one of several configuration files.  Like
+other configuration values, you can set it in ``bazaar.conf`` as a
+general setting.  If you want to override the value for a particular
+branch, or set of branches, you can use ``locations.conf``.
+``.bzr/branch/branch.conf`` will also work, but will cause all commits
+to that branch to use the same email address, even if someone else
+does them.
 
 The order of precedence is
 
- 1. If the `$BZREMAIL` env variable is set, use it.
- 2. If an email is set for your current branch in the `locations.conf` ini
+ 1. If the ``$BZR_EMAIL`` environment variable is set.
+ #. If an email is set for your current branch in the ``locations.conf``
     file.
- 3. If an email is set in the `bazaar.conf` default ini file.
- 4. If the `$EMAIL` env variable is set.
- 5. Bazaar will try to guess based on your username and the hostname.
+ #. If an email is set four your current branch in the
+    ``.bzr/branch/branch.conf`` file.
+ #. If an email is set in the ``bazaar.conf`` default configuration file.
+ #. If the `$EMAIL` environment variable is set.
+ #. Bazaar will try to guess based on your username and the hostname.
 
-To check on what Bazaar thinks your current email is, use the '''whoami'''
+To check on what Bazaar thinks your current email is, use the ``whoami``
 ("who am i?") command::
 
     % bzr whoami
@@ -49,14 +55,15 @@
 
     % bzr whoami --branch 'Joe Cool <joe@example.com>'
 
-These modify your global bazaar.conf or branch branch.conf file, respectively.
+These modify your global ``bazaar.conf`` or branch ``branch.conf`` file, respectively.
 
-Setting email via default ini file
-==================================
-To use the default ini file, create the file `$HOME/.bazaar/bazaar.conf` (on
-Windows this is `%APPDATA%\bazaar\2.0\bazaar.conf`) and set an email address
+Setting email via default configuration file
+============================================
+To use the default ini file, create the file ``$HOME/.bazaar/bazaar.conf`` (on
+Windows this is ``%APPDATA%\bazaar\2.0\bazaar.conf``) and set an email address
 as shown below.  Please note that the word DEFAULT is case sensitive, and
-must be in upper-case.::
+must be in upper-case.
+::
 
     [DEFAULT]
     email=Your Name <name@isp.com>
@@ -67,29 +74,29 @@
 Setting email on a per-branch basis
 ===================================
 
-The second approach is to set email on a branch by branch basis by using the
-`$HOME/.bazaar/locations.conf` ini file, thusly::
+The second approach is to set email on a branch by branch basis by
+using the ``$HOME/.bazaar/locations.conf`` configuration file,
+thusly::
 
     [/some/branch/location]
     email=Your Name <name@other-isp.com>
 
-This will set your email address in the branch at `/some/branch/location`,
-overriding the default specified in the `bazaar.conf` above.
-
-Setting email via environment
------------------------------
-The final method Bazaar will use is checking for the `$BZREMAIL` and
-`$EMAIL` environment variables.  Generally, one would use this method to
-override the email in a script context.  If you would like to set a general
-default, then please see the ini methods above.
-
-
-Concerns for spam
-=================
+This will set your email address in the branch at ``/some/branch/location``,
+overriding the default specified in the ``bazaar.conf`` above.
+
+Setting email via environment variable
+--------------------------------------
+The final method Bazaar will use is checking for the ``$BZR_EMAIL``
+and ``$EMAIL`` environment variables.  Generally, you would use this
+method to override the email in a script context.  If you would like
+to set a general default, then please see the ini methods above.
+
+Concerns about spam
+===================
 Some people want to avoid sharing their email address so as not to get
 spam.  Bazaar will never disclose your email address, unless you publish
 a branch or changeset in a public location.  It's recommended that you
-''do'' use a real address, so that people can contact you about your work,
+*do* use a real address, so that people can contact you about your work,
 but it's not required.  You can use an address which is obfuscated, which
 bounces, or which goes through an anti-spam service such as
 `spamgourmet.com`.

=== modified file 'doc/specifying_revisions.txt'
--- doc/specifying_revisions.txt	2006-10-11 22:43:03 +0000
+++ doc/specifying_revisions.txt	2007-03-02 12:40:45 +0000
@@ -3,15 +3,15 @@
 =============================
 
 Description of the subarguments for ``-r`` as used by commands such as
-**bzr log** and **bzr merge**.
+``bzr log`` and ``bzr merge``.
 
 You can get the online documentation with ``bzr help revisionspec``.
 
 Revision specs and ranges
 =========================
 
-**bzr** has a very expressive way to specify a revision, or a range of revisions.
-We'll take the example of the **log** command.
+``bzr`` has a very expressive way to specify a revision, or a range of revisions.
+We'll take the example of the ``log`` command.
 
 To specify a range of revisions, use for example::
 
@@ -22,10 +22,10 @@
     $ bzr log -r 1..
     $ bzr log -r ..4
 
-Note::
-    Omitting the lower bound doesn't work on currently released versions.
+Note:
+    Omitting the lower bound doesn't work on versions of ``bzr`` prior to 0.14.
 
-Other commands, like **bzr cat** take only one revision, not a range, like::
+Other commands, like ``bzr cat`` take only one revision, not a range, like::
 
     $ bzr cat -r 42 foo.c
 
@@ -58,7 +58,7 @@
 -------
 
 Positive numbers denote revision numbers in the current branch. Revision
-numbers are labelled as "revno" in the output of **bzr log**.  To display
+numbers are labelled as "revno" in the output of ``bzr log``.  To display
 the log for the first ten revisions::
 
     $ bzr log -r ..10
@@ -82,8 +82,8 @@
 revid
 -----
 
-**revid** allows specifying a an internal revision ID, as show by **bzr
-testament** and some other commands.
+**revid** allows specifying a an internal revision ID, as shown by ``bzr
+log`` and some other commands.
 
 For example::
 
@@ -98,7 +98,7 @@
     that appears before ''rev'' in the revision history, or the revision that
     was current when ''rev'' what comitted.
 
-''rev'' can be any revision specifier.
+''rev'' can be any revision specifier and may be chained.
 
 For example::
 
@@ -131,14 +131,14 @@
 --------
 
 **ancestor**:*path*
-    specifies the common ancestor between the current branch,and a 
+    specifies the common ancestor between the current branch and a 
     different branch. This is the same ancestor that would be used for
     merging purposes.
 
 *path* may be the URL of a remote branch, or the file path to a local branch.
 
 For example, to see what changes were made on a branch since it was forked
-off **../parent**::
+off ``../parent``::
 
     $ bzr diff -r ancestor:../parent
 
@@ -146,9 +146,9 @@
 ------
 
 branch
-   ''path'' specifies the latest revision in another branch.
+   ``path`` specifies the latest revision in another branch.
 
-''path'' may be the URL of a remote branch, or the file path to a local branch.
+``path`` may be the URL of a remote branch, or the file path to a local branch.
 
 For example, to get the differences between this and another branch::
 

=== modified file 'doc/tutorial.txt'
--- doc/tutorial.txt	2006-11-11 19:21:47 +0000
+++ doc/tutorial.txt	2007-03-02 12:40:45 +0000
@@ -69,8 +69,8 @@
 
 The centralized model can have some drawbacks. A centralized RCS requires
 that one is able to connect to the server whenever one wants to do version
-control work. This can be a bit of a problem if your server on some other
-machine on the internet and you are not. Or, worse yet, you ''are'' on the
+control work. This can be a bit of a problem if your server is on some other
+machine on the internet and you are not. Or, worse yet, you **are** on the
 internet but the server is missing!
 
 Decentralized Revision Control Systems (which I'll call DRCS after this
@@ -90,11 +90,11 @@
 a **revision** of the directory tree whenever the user asks. 
 
 Revision control software such as Bazaar can do much more than just
-storage and performing undo.  For example, with Bazaar developer can
-take the modifications in one branch of software and apply them to
-another, related, branch -- even if those changes exist in a branch owned
-by somebody else. This allows developers to cooperate without giving write
-access to repository.
+storage and performing undo.  For example, with Bazaar a developer can
+take the modifications in one branch of software and apply them to a
+related branch -- even if those changes exist in a branch owned by
+somebody else. This allows developers to cooperate without giving
+write access to the repository.
 
 Bazaar remembers the ''ancestry'' of a revision: the previous revisions
 that it is based upon.  A single revision may have more than one direct
@@ -119,32 +119,31 @@
 guess that Bazaar makes, then three options exist:
 
  1. Set an email address via ``bzr whoami``.  This is the simplest way.
- To set a global identity, use::
- 
-     % bzr whoami 'Your Name <email@example.com>'
- 
- If you'd like to use a different address for a specific branch, enter
- the branch folder and use::
- 
-     % bzr whoami --branch 'Your Name <email@example.com>'
- 
- 1. Setting the email address in the
- ``~/.bazaar/bazaar.conf`` [1]_ by adding the following lines. Please note that
- ``[DEFAULT]`` is case sensitive::
 
+    To set a global identity, use::
+ 
+    % bzr whoami 'Your Name <email@example.com>'
+ 
+    If you'd like to use a different address for a specific branch, enter
+    the branch folder and use::
+ 
+    % bzr whoami --branch 'Your Name <email@example.com>'
+ 
+ #. Setting the email address in the ``~/.bazaar/bazaar.conf`` [1]_ by adding the following lines.
+    Please note that  ``[DEFAULT]`` is case sensitive::
     [DEFAULT]
     email= Your Name <email@isp.com>
  
- As above, you can override this settings on a branch by branch basis by
- creating a branch section in ``~/.bazaar/locations.conf`` and adding the
- following lines::
-
-    [/the/directory/to/the/branch]
+    As above, you can override this settings on a branch by branch basis by
+    creating a branch section in ``~/.bazaar/locations.conf`` and adding the
+    following lines::
+    [/the/path/to/the/branch]
     email=Your Name <email@isp.com>
- 
- 1. Overriding the two previous options by setting the global environment
- variable ``$BZREMAIL`` or ``$EMAIL`` (``$BZREMAIL`` will take precedence) 
- to your full email address.
+
+
+ #. Overriding the two previous options by setting the global environment
+    variable ``$BZREMAIL`` or ``$EMAIL`` (``$BZREMAIL`` will take precedence) 
+    to your full email address.
 
 .. [1] On Windows, the users configuration files can be found in the
    application data directory. So instead of ``~/.bazaar/branch.conf``
@@ -156,10 +155,10 @@
 Creating a branch
 =================
 
-History is by default stored in the .bzr directory of the branch. There
-will be a facility to store it in a separate repository, which may be
-remote.  We create a new branch by running **bzr init** in an existing
-directory::
+History is by default stored in the .bzr directory of the branch. In a
+future version of Bazaar, there will be a facility to store it in a
+separate repository, which may be remote.  We create a new branch by
+running ``bzr init`` in an existing directory::
 
     % mkdir tutorial
     % cd tutorial
@@ -173,7 +172,7 @@
     ./  ../  .bzr/
     %
 
-As for CVS, there are three classes of file: unknown, ignored, and
+As with CVS, there are three classes of file: unknown, ignored, and
 versioned.  The **add** command makes a file versioned: that is, changes
 to it will be recorded by the system::
 
@@ -198,17 +197,18 @@
 containing control files.  By default there is no separate repository or
 database as used in svn or svk. You can choose to create a repository if
 you want to (see the **bzr init-repo** command). You may wish to do this
-if you have very large branches, or many branches of a moderate sized
+if you have very large branches, or many branches of a moderately sized
 project.
 
 You'll usually refer to branches on your computer's filesystem just by
 giving the name of the directory containing the branch.  bzr also supports
-accessing branches over http, for example::
+accessing branches over http and sftp, for example::
 
     % bzr log http://bazaar-vcs.org/bzr/bzr.dev/
+    % bzr log sftp://bazaar-vcs.org/bzr/bzr.dev/
 
-By installing bzr plugins you can also access branches over the sftp or
-rsync protocols.
+By installing bzr plugins you can also access branches using the rsync
+protocol.
 
 Reviewing changes
 =================
@@ -499,14 +499,23 @@
   % bzr merge URL
 
 
-If there is a conflict during a merge, 3 files with the same basename are
-created. The filename of the common base is appended with .BASE, the
-filename of the file containing your changes is appended .THIS and the
-filename with the changes from the other tree is appended .OTHER.
-Using a program such as kdiff3, you can now comfortably merge them into
-one file. To commit you have to rename it to the original basename and
-delete the other two files. As long as there exist files with .BASE, .THIS
-or .OTHER the commit command will complain.
+If there is a conflict during a merge, 3 files with the same basename
+are created. The filename of the common base is appended with ".BASE",
+the filename of the file containing your changes is appended with
+".THIS" and the filename with the changes from the other tree is
+appended with ".OTHER".  Using a program such as kdiff3, you can now
+comfortably merge them into one file.  In order to commit you have to
+rename the merged file (".THIS") to the original file name.  To
+complete the conflict resolution you must use the resolve command,
+which will remove the ".OTHER" and ".BASE" files.  As long as there
+exist files with .BASE, .THIS or .OTHER the commit command will
+report an error.
+
+::
+
+  % kdiff3 file.BASE file.OTHER file.THIS
+  % mv file.THIS file
+  % bzr resolve file
 
 [**TODO**: explain conflict markers within files]
 
@@ -519,13 +528,14 @@
 directory.  One can push a branch (or the changes for a branch) by one of
 the following three methods:
 
-* Rsync:   rsync -avrz LOCALBRANCH  servername.com/this/directory/here
+* Rsync:   rsync -avrz LOCALBRANCH  servername.com/path/to/directory
 
   (or any other tool for publishing a directory to a web site.)
 
-* bzr push sftp://servername.com/this/directory/here 
+* bzr push sftp://servername.com/path/to/directory 
 
-  (The directory that must already exist)
+  (The destination directory must already exist unless the
+  ``--create-prefix`` option is used.)
 
 * The rspush plugin that comes with BzrTools
 

=== modified file 'doc/using_aliases.txt'
--- doc/using_aliases.txt	2006-07-30 06:52:39 +0000
+++ doc/using_aliases.txt	2007-03-02 12:40:45 +0000
@@ -6,8 +6,8 @@
 are an easy way to create shortcuts for commonly-typed commands, or to set
 defaults for commands.
 
-Command aliases can be defined in the **[ALIASES]** section of your
-**.bazaar/bazaar.conf** file. Aliases start with the alias name, then an
+Command aliases can be defined in the ``[ALIASES]`` section of your
+``.bazaar/bazaar.conf`` file. Aliases start with the alias name, then an
 equal sign, then a command fragment.  Here's an example ALIASES section:
 
 Examples
@@ -23,29 +23,39 @@
 Explanation of examples
 =======================
 
- * The first alias makes a new 'recentlog' command that shows the logs for the
+ * The first alias makes a new ``recentlog`` command that shows the logs for the
    last three revisions
- * The **ll** alias shows the last 10 log entries in line format.
- * the **commit** alias sets the default for commit to refuse to commit if new
+ * The ``ll`` alias shows the last 10 log entries in line format.
+ * the ``commit`` alias sets the default for commit to refuse to commit if new
    files in the tree are not recognized.
- * the **diff** alias adds the coveted -p option to diff
-
+ * the ``diff`` alias adds the coveted -p option to diff
+
+Using the Aliases
+=================
+The aliases defined above would be used like so: ::
+
+   % bzr recentlog
+   % bzr ll
+   % bzr commit
+   % bzr diff
 
 Rules for Aliases
 =================
 
- * You can override parts of the options given in an alias by overriding it.
-   For example, if you run **lastlog -r-5..**, you will only get five
-   line-based log entries instead of 10.
+ * You can override a portion of the options given in an alias by
+   specifying the new part on the command-line.  For example, if
+   you run ``lastlog -r-5..``, you will only get five line-based log
+   entries instead of 10.  Note that all boolean options have an
+   implicit inverse, so you can override the commit alias with
+   ``commit --no-strict``.
+
  * Aliases can override the standard behaviour of existing commands by giving
-   an alias name that is thesame as the orignal command. For example, default
-   commit is changed with **commit=commit --strict**.
- * Aliases can not refer to other aliases. In other words making a
-   **lastlog** alias and referring to it with a **ll** alias will not work.
-
-This includes aliases that override standard commands
-
- * Giving the **--no-aliases** to the bzr command will tell it to ignore aliases
-   for that run. For example, running **bzr --no-aliases commit** will perform a
-   standard commit instead not do a **commit --strict**.
+   an alias name that is the same as the orignal command. For example, default
+   commit is changed with ``commit=commit --strict``.
+ * Aliases cannot refer to other aliases. In other words making a
+   ``lastlog`` alias and referring to it with a ``ll`` alias will not work.
+   This includes aliases that override standard commands
+ * Giving the ``--no-aliases`` to the bzr command will tell it to ignore aliases
+   for that run. For example, running ``bzr --no-aliases commit`` will perform a
+   standard commit instead not do a ``commit --strict``.
 

=== modified file 'doc/version_info.txt'
--- doc/version_info.txt	2006-09-21 22:12:28 +0000
+++ doc/version_info.txt	2007-02-27 22:14:14 +0000
@@ -6,7 +6,7 @@
 ========
 
 This document describes ways of using ``bzr version-info`` as part of a
-build routine to embed version information into a final project.
+build process to embed version information into a final project.
 
 
 Python Project
@@ -23,24 +23,22 @@
 
 This generates a file which contains 3 dictionaries:
 
-  `version_info`: A dictionary containing the basic information about the
-                  current state.
-
-  revisions: A dictionary listing all of the revisions in the history of
-             the tree, along with the commit times and commit message.
-             This defaults to being empty unless ``--all`` or
-             ``--include-history`` is supplied. This is useful if you
-             want to track what bugfixes, etc, might be included in the
-             released version. But for many projects it is more
-             information than they need.
-
-  `file_revisions`: A dictionary listing the last-modified revision for
-                    all files in the project. This can be used similarly
-                    to how ``$Id$`` keywords are used in CVS controlled
-                    files. The last modified date can be determined by
-                    looking in the ``revisions`` map. This is also empty by
-                    default, and enabled only by ``--all`` or
-                    ``--include-file-revisions``.
+  * `version_info`: A dictionary containing the basic information about the
+    current state.
+
+  * `revisions`: A dictionary listing all of the revisions in the
+    history of the tree, along with the commit times and commit
+    message.  This defaults to being empty unless ``--all`` or
+    ``--include-history`` is supplied. This is useful if you want to
+    track what bug fixes, etc, might be included in the released
+    version. But for many projects it is more information than needed.
+
+  * `file_revisions`: A dictionary listing the last-modified revision
+    for all files in the project. This can be used similarly to how
+    ``$Id$`` keywords are used in CVS-controlled files. The last
+    modified date can be determined by looking in the ``revisions``
+    map. This is also empty by default, and enabled only by ``--all``
+    or ``--include-file-revisions``.
 
 Check Clean
 ===========

